@@ -34,10 +34,7 @@ G_BEGIN_DECLS
 #define GDK_IS_DEVICE(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GDK_TYPE_DEVICE))
 
 typedef struct _GdkDevice GdkDevice;
-typedef struct _GdkDevicePrivate GdkDevicePrivate;
-
-typedef struct _GdkDeviceKey GdkDeviceKey;
-typedef struct _GdkDeviceAxis GdkDeviceAxis;
+typedef struct _GdkDeviceClass GdkDeviceClass;
 typedef struct _GdkTimeCoord GdkTimeCoord;
 
 /**
@@ -141,37 +138,6 @@ typedef enum {
   GDK_DEVICE_TYPE_FLOATING
 } GdkDeviceType;
 
-/**
- * GdkDeviceKey:
- * @keyval: the keyval to generate when the macro button is pressed.
- *          If this is 0, no keypress will be generated.
- * @modifiers: the modifiers set for the generated key event.
- *
- * The <structname>GdkDeviceKey</structname> structure contains information
- * about the mapping of one device macro button onto a normal X key event.
- */
-struct _GdkDeviceKey
-{
-  guint keyval;
-  GdkModifierType modifiers;
-};
-
-/**
- * GdkDeviceAxis:
- * @use: specifies how the axis is used.
- * @min: the minimal value that will be reported by this axis.
- * @max: the maximal value that will be reported by this axis.
- *
- * The <structname>GdkDeviceAxis</structname> structure contains information
- * about the range and mapping of a device axis.
- */
-struct _GdkDeviceAxis
-{
-  GdkAxisUse use;
-  gdouble    min;
-  gdouble    max;
-};
-
 /* We don't allocate each coordinate this big, but we use it to
  * be ANSI compliant and avoid accessing past the defined limits.
  */
@@ -190,26 +156,6 @@ struct _GdkTimeCoord
   gdouble axes[GDK_MAX_TIMECOORD_AXES];
 };
 
-struct _GdkDevice
-{
-  GObject parent_instance;
-
-  /* All fields are read-only */
-  gchar *GSEAL (name);
-  GdkInputSource GSEAL (source);
-  GdkInputMode GSEAL (mode);
-  gboolean GSEAL (has_cursor);	     /* TRUE if a X pointer follows device motion */
-
-  gint GSEAL (num_axes);
-  GdkDeviceAxis *GSEAL (axes);
-
-  gint GSEAL (num_keys);
-  GdkDeviceKey *GSEAL (keys);
-
-  /*< private >*/
-  GdkDevicePrivate *priv;
-};
-
 GType gdk_device_get_type (void) G_GNUC_CONST;
 
 G_CONST_RETURN gchar *gdk_device_get_name       (GdkDevice *device);
@@ -218,20 +164,21 @@ gboolean              gdk_device_get_has_cursor (GdkDevice *device);
 /* Functions to configure a device */
 GdkInputSource gdk_device_get_source    (GdkDevice      *device);
 void           gdk_device_set_source    (GdkDevice      *device,
-					 GdkInputSource  source);
+                                         GdkInputSource  source);
 
 GdkInputMode   gdk_device_get_mode      (GdkDevice      *device);
 gboolean       gdk_device_set_mode      (GdkDevice      *device,
-					 GdkInputMode    mode);
+                                         GdkInputMode    mode);
 
+gint           gdk_device_get_n_keys    (GdkDevice       *device);
 gboolean       gdk_device_get_key       (GdkDevice       *device,
                                          guint            index_,
                                          guint           *keyval,
                                          GdkModifierType *modifiers);
 void           gdk_device_set_key       (GdkDevice      *device,
-					 guint           index_,
-					 guint           keyval,
-					 GdkModifierType modifiers);
+                                         guint           index_,
+                                         guint           keyval,
+                                         GdkModifierType modifiers);
 
 GdkAxisUse     gdk_device_get_axis_use  (GdkDevice         *device,
                                          guint              index_);
@@ -241,19 +188,19 @@ void           gdk_device_set_axis_use  (GdkDevice         *device,
 
 
 void     gdk_device_get_state    (GdkDevice         *device,
-				  GdkWindow         *window,
-				  gdouble           *axes,
-				  GdkModifierType   *mask);
+                                  GdkWindow         *window,
+                                  gdouble           *axes,
+                                  GdkModifierType   *mask);
 gboolean gdk_device_get_history  (GdkDevice         *device,
-				  GdkWindow         *window,
-				  guint32            start,
-				  guint32            stop,
-				  GdkTimeCoord    ***events,
-				  guint             *n_events);
+                                  GdkWindow         *window,
+                                  guint32            start,
+                                  guint32            stop,
+                                  GdkTimeCoord    ***events,
+                                  gint              *n_events);
 void     gdk_device_free_history (GdkTimeCoord     **events,
-				  gint               n_events);
+                                  gint               n_events);
 
-guint    gdk_device_get_n_axes     (GdkDevice       *device);
+gint     gdk_device_get_n_axes     (GdkDevice       *device);
 GList *  gdk_device_list_axes      (GdkDevice       *device);
 gboolean gdk_device_get_axis_value (GdkDevice       *device,
                                     gdouble         *axes,
@@ -261,12 +208,13 @@ gboolean gdk_device_get_axis_value (GdkDevice       *device,
                                     gdouble         *value);
 
 gboolean gdk_device_get_axis     (GdkDevice         *device,
-				  gdouble           *axes,
-				  GdkAxisUse         use,
-				  gdouble           *value);
+                                  gdouble           *axes,
+                                  GdkAxisUse         use,
+                                  gdouble           *value);
 GdkDisplay * gdk_device_get_display (GdkDevice      *device);
 
 GdkDevice  * gdk_device_get_associated_device (GdkDevice     *device);
+GList *      gdk_device_list_slave_devices    (GdkDevice     *device);
 
 GdkDeviceType gdk_device_get_device_type (GdkDevice *device);
 
@@ -281,6 +229,11 @@ GdkGrabStatus gdk_device_grab        (GdkDevice        *device,
 void          gdk_device_ungrab      (GdkDevice        *device,
                                       guint32           time_);
 
+void          gdk_device_warp        (GdkDevice        *device,
+                                      GdkScreen        *screen,
+                                      gint              x,
+                                      gint              y);
+
 gboolean gdk_device_grab_info_libgtk_only (GdkDisplay  *display,
                                            GdkDevice   *device,
                                            GdkWindow  **grab_window,
