@@ -18,9 +18,10 @@
  */
 
 #include "config.h"
-#include "gdkdevicemanager.h"
+
+#include "gdkdevicemanagerprivate.h"
+#include "gdkdisplay.h"
 #include "gdkintl.h"
-#include "gdkinternals.h"
 
 
 /**
@@ -34,12 +35,13 @@
  *                    additional features such as sub-pixel positioning information and additional
  *                    device-dependent information.
  * @Title: GdkDeviceManager
- * @See_also: #GdkDevice, #GdkEvent, gdk_enable_multidevice()
+ * @See_also: #GdkDevice, #GdkEvent, gdk_disable_multidevice()
  *
- * By default, GDK supports the traditional single keyboard/pointer input scheme (Plus additional
- * special input devices such as tablets. In short, backwards compatible with 2.X). Since version 3.0,
- * if gdk_enable_multidevice() is called before gdk_display_open() and the platform supports it, GDK
- * will be aware of multiple keyboard/pointer pairs interacting simultaneously with the user interface.
+ * By default, and if the platform supports it, GDK is aware of multiple keyboard/pointer pairs
+ * and multitouch devices, this behavior can be changed by calling gdk_disable_multidevice()
+ * before gdk_display_open(), although there would be rarely a reason to do that. For a widget
+ * or window to be dealt as multipointer aware, gdk_window_set_support_multidevice() or
+ * gtk_widget_set_support_multidevice() must have been called on it.
  *
  * Conceptually, in multidevice mode there are 2 device types, virtual devices (or master devices)
  * are represented by the pointer cursors and keyboard foci that are seen on the screen. physical
@@ -82,8 +84,8 @@
  *
  * In order to query the device hierarchy and be aware of changes in the device hierarchy (such as
  * virtual devices being created or removed, or physical devices being plugged or unplugged), GDK
- * provides #GdkDeviceManager. On X11, multidevice support is implemented through XInput 2. If
- * gdk_enable_multidevice() is called, the XInput 2.x #GdkDeviceManager implementation will be used
+ * provides #GdkDeviceManager. On X11, multidevice support is implemented through XInput 2. Unless
+ * gdk_disable_multidevice() is called, the XInput 2.x #GdkDeviceManager implementation will be used
  * as input source, else either the core or XInput 1.x implementations will be used.
  */
 
@@ -114,12 +116,6 @@ enum {
 static guint signals [LAST_SIGNAL] = { 0 };
 
 
-struct _GdkDeviceManagerPrivate
-{
-  GdkDisplay *display;
-};
-
-
 static void
 gdk_device_manager_class_init (GdkDeviceManagerClass *klass)
 {
@@ -180,12 +176,16 @@ gdk_device_manager_class_init (GdkDeviceManagerClass *klass)
    * @device_manager: the object on which the signal is emitted
    * @device: the #GdkDevice that changed.
    *
-   * The ::device-changed signal is emitted either when some
-   * #GdkDevice has changed the number of either axes or keys.
-   * For example In X this will normally happen when the slave
-   * device routing events through the master device changes,
-   * in that case the master device will change to reflect the
-   * new slave device axes and keys.
+   * The ::device-changed signal is emitted whenever a device
+   * has changed in the hierarchy, either slave devices being
+   * disconnected from their master device or connected to
+   * another one, or master devices being added or removed
+   * a slave device.
+   *
+   * If a slave device is detached from all master devices
+   * (gdk_device_get_associated_device() returns %NULL), its
+   * #GdkDeviceType will change to %GDK_DEVICE_TYPE_FLOATING,
+   * if it's attached, it will change to %GDK_DEVICE_TYPE_SLAVE.
    */
   signals [DEVICE_CHANGED] =
     g_signal_new (g_intern_static_string ("device-changed"),
@@ -196,18 +196,11 @@ gdk_device_manager_class_init (GdkDeviceManagerClass *klass)
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   GDK_TYPE_DEVICE);
-
-  g_type_class_add_private (object_class, sizeof (GdkDeviceManagerPrivate));
 }
 
 static void
 gdk_device_manager_init (GdkDeviceManager *device_manager)
 {
-  GdkDeviceManagerPrivate *priv;
-
-  device_manager->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (device_manager,
-                                                             GDK_TYPE_DEVICE_MANAGER,
-                                                             GdkDeviceManagerPrivate);
 }
 
 static void
@@ -216,14 +209,10 @@ gdk_device_manager_set_property (GObject      *object,
                                  const GValue *value,
                                  GParamSpec   *pspec)
 {
-  GdkDeviceManagerPrivate *priv;
-
-  priv = GDK_DEVICE_MANAGER (object)->priv;
-
   switch (prop_id)
     {
     case PROP_DISPLAY:
-      priv->display = g_value_get_object (value);
+      GDK_DEVICE_MANAGER (object)->display = g_value_get_object (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -237,14 +226,11 @@ gdk_device_manager_get_property (GObject      *object,
                                  GValue       *value,
                                  GParamSpec   *pspec)
 {
-  GdkDeviceManagerPrivate *priv;
-
-  priv = GDK_DEVICE_MANAGER (object)->priv;
 
   switch (prop_id)
     {
     case PROP_DISPLAY:
-      g_value_set_object (value, priv->display);
+      g_value_set_object (value, GDK_DEVICE_MANAGER (object)->display);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -258,21 +244,18 @@ gdk_device_manager_get_property (GObject      *object,
  *
  * Gets the #GdkDisplay associated to @device_manager.
  *
- * Returns: the #GdkDisplay to which @device_manager is
- *          associated to, or #NULL.
+ * Returns: (transfer none): the #GdkDisplay to which @device_manager is
+ *          associated to, or #NULL. This memory is owned by GDK and
+ *          must not be freed or unreferenced.
  *
  * Since: 3.0
  **/
 GdkDisplay *
 gdk_device_manager_get_display (GdkDeviceManager *device_manager)
 {
-  GdkDeviceManagerPrivate *priv;
-
   g_return_val_if_fail (GDK_IS_DEVICE_MANAGER (device_manager), NULL);
 
-  priv = device_manager->priv;
-
-  return priv->display;
+  return device_manager->display;
 }
 
 /**
@@ -283,7 +266,8 @@ gdk_device_manager_get_display (GdkDeviceManager *device_manager)
  * Returns the list of devices of type @type currently attached to
  * @device_manager.
  *
- * Returns: a list of #GdkDevice<!-- -->s. The returned list must be
+ * Returns: (transfer container) (element-type Gdk.Device): a list of 
+ *          #GdkDevice<!-- -->s. The returned list must be
  *          freed with g_list_free (). The list elements are owned by
  *          GTK+ and must not be freed or unreffed.
  *
@@ -309,7 +293,8 @@ gdk_device_manager_list_devices (GdkDeviceManager *device_manager,
  * You should use this function sheldomly, only in code that isn't triggered by a #GdkEvent
  * and there aren't other means to get a meaningful #GdkDevice to operate on.
  *
- * Returns: The client pointer.
+ * Returns: (transfer none): The client pointer. This memory is
+ *          owned by GDK and must not be freed or unreferenced.
  *
  * Since: 3.0
  **/
