@@ -124,7 +124,7 @@ static NSAutoreleasePool *autorelease_pool;
  * a run loop iteration, so we need to detect that and avoid triggering
  * our "run the GLib main looop while the run loop is active machinery.
  */
-static gint getting_events = 0;
+static gboolean getting_events;
 
 /************************************************************
  *********              Select Thread               *********
@@ -632,21 +632,21 @@ gdk_event_check (GSource *source)
 
   GDK_THREADS_ENTER ();
 
-  /* Refresh the autorelease pool if we're at the base CFRunLoop level
-   * (indicated by current_loop_level) and the base g_main_loop level
-   * (indicated by g_main_depth()). Messing with the autorelease pool at
-   * any level of nesting can cause access to deallocated memory because
-   * autorelease_pool is static and releasing a pool will cause all pools
-   * allocated inside of it to be released as well.
+  /* XXX: This check isn't right it won't handle a recursive GLib main
+   * loop run within an outer CFRunLoop run. Such loops will pile up
+   * memory. Fixing this requires setting a flag *only* when we call
+   * g_main_context_check() from within the run loop iteraton code,
+   * and also maintaining our own stack of run loops... allocating and
+   * releasing NSAutoReleasePools not properly nested with CFRunLoop
+   * runs seems to cause problems.
    */
-  if (current_loop_level == 0 && g_main_depth() == 0)
+  if (current_loop_level == 0)
     {
       if (autorelease_pool)
-        [autorelease_pool drain];
-
+	[autorelease_pool release];
       autorelease_pool = [[NSAutoreleasePool alloc] init];
     }
-
+  
   retval = (_gdk_event_queue_find_first (_gdk_display) != NULL ||
 	    _gdk_quartz_event_loop_check_pending ());
 
@@ -670,8 +670,7 @@ gdk_event_dispatch (GSource     *source,
 
   if (event)
     {
-      if (_gdk_event_func)
-	(*_gdk_event_func) (event, _gdk_event_data);
+      _gdk_event_emit (event);
 
       gdk_event_free (event);
     }
@@ -712,12 +711,12 @@ poll_func (GPollFD *ufds,
   else
     limit_date = [NSDate dateWithTimeIntervalSinceNow:timeout_/1000.0];
 
-  getting_events++;
+  getting_events = TRUE;
   event = [NSApp nextEventMatchingMask: NSAnyEventMask
 	                     untilDate: limit_date
 	                        inMode: NSDefaultRunLoopMode
                                dequeue: YES];
-  getting_events--;
+  getting_events = FALSE;
 
   if (n_ready < 0)
     n_ready = select_thread_collect_poll (ufds, nfds);
@@ -778,6 +777,8 @@ query_main_context (GMainContext *context,
 static void
 run_loop_entry (void)
 {
+  current_loop_level++;
+
   if (acquired_loop_level == -1)
     {
       if (g_main_context_acquire (NULL))
@@ -926,13 +927,16 @@ run_loop_after_waiting (void)
 static void
 run_loop_exit (void)
 {
-  /* + 1 because we decrement current_loop_level separately in observer_callback() */
-  if ((current_loop_level + 1) == acquired_loop_level)
+  g_return_if_fail (current_loop_level > 0);
+
+  if (current_loop_level == acquired_loop_level)
     {
       g_main_context_release (NULL);
       acquired_loop_level = -1;
       GDK_NOTE (EVENTLOOP, g_print ("EventLoop: Ended tracking run loop activity\n"));
     }
+  
+  current_loop_level--;
 }
 
 static void
@@ -940,22 +944,9 @@ run_loop_observer_callback (CFRunLoopObserverRef observer,
 			    CFRunLoopActivity    activity,
 			    void                *info)
 {
-  switch (activity)
-    {
-    case kCFRunLoopEntry:
-      current_loop_level++;
-      break;
-    case kCFRunLoopExit:
-      g_return_if_fail (current_loop_level > 0);
-      current_loop_level--;
-      break;
-    default:
-      break;
-    }
-
-  if (getting_events > 0) /* Activity we triggered */
+  if (getting_events) /* Activity we triggered */
     return;
-
+  
   switch (activity)
     {
     case kCFRunLoopEntry:
