@@ -32,10 +32,16 @@
 #include "gdkscreen.h"
 #include "gdkkeysyms.h"
 #include "gdkprivate-quartz.h"
+#include "gdkdevicemanager-core.h"
 
 #define GRIP_WIDTH 15
 #define GRIP_HEIGHT 15
 
+#define WINDOW_IS_TOPLEVEL(window)		     \
+  (GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&   \
+   GDK_WINDOW_TYPE (window) != GDK_WINDOW_FOREIGN && \
+   GDK_WINDOW_TYPE (window) != GDK_WINDOW_OFFSCREEN)
+
 /* This is the window corresponding to the key window */
 static GdkWindow   *current_keyboard_window;
 
@@ -68,77 +74,45 @@ gdk_events_pending (void)
 	  (_gdk_quartz_event_loop_check_pending ()));
 }
 
-GdkEvent*
-gdk_event_get_graphics_expose (GdkWindow *window)
-{
-  /* FIXME: Implement */
-  return NULL;
-}
-
-GdkGrabStatus
-gdk_keyboard_grab (GdkWindow  *window,
-		   gint        owner_events,
-		   guint32     time)
-{
-  GdkDisplay *display;
-  GdkWindow  *toplevel;
-
-  g_return_val_if_fail (window != NULL, 0);
-  g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
-
-  display = gdk_drawable_get_display (window);
-  toplevel = gdk_window_get_toplevel (window);
-
-  _gdk_display_set_has_keyboard_grab (display,
-                                      window,
-                                      toplevel,
-                                      owner_events,
-                                      0,
-                                      time);
-
-  return GDK_GRAB_SUCCESS;
-}
-
 void
-gdk_display_keyboard_ungrab (GdkDisplay *display,
-			     guint32     time)
+gdk_device_ungrab (GdkDevice *device,
+                   guint32    time_)
 {
-  _gdk_display_unset_has_keyboard_grab (display, FALSE);
-}
+  GdkDeviceGrabInfo *grab;
 
-void
-gdk_display_pointer_ungrab (GdkDisplay *display,
-			    guint32     time)
-{
-  GdkPointerGrabInfo *grab;
-
-  grab = _gdk_display_get_last_pointer_grab (display);
+  grab = _gdk_display_get_last_device_grab (_gdk_display, device);
   if (grab)
     grab->serial_end = 0;
 
-  _gdk_display_pointer_grab_update (display, 0);
+  _gdk_display_device_grab_update (_gdk_display, device, 0);
 }
 
 GdkGrabStatus
-_gdk_windowing_pointer_grab (GdkWindow    *window,
-                             GdkWindow    *native,
-                             gboolean	   owner_events,
-                             GdkEventMask  event_mask,
-                             GdkWindow    *confine_to,
-                             GdkCursor    *cursor,
-                             guint32       time)
+_gdk_windowing_device_grab (GdkDevice    *device,
+                            GdkWindow    *window,
+                            GdkWindow    *native,
+                            gboolean      owner_events,
+                            GdkEventMask  event_mask,
+                            GdkWindow    *confine_to,
+                            GdkCursor    *cursor,
+                            guint32       time)
 {
   g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
   g_return_val_if_fail (confine_to == NULL || GDK_IS_WINDOW (confine_to), 0);
 
-  _gdk_display_add_pointer_grab (_gdk_display,
-                                 window,
-                                 native,
-                                 owner_events,
-                                 event_mask,
-                                 0,
-                                 time,
-                                 FALSE);
+  if (!window || GDK_WINDOW_DESTROYED (window))
+    return GDK_GRAB_NOT_VIEWABLE;
+
+  _gdk_display_add_device_grab (_gdk_display,
+                                device,
+                                window,
+                                native,
+                                GDK_OWNERSHIP_NONE,
+                                owner_events,
+                                event_mask,
+                                0,
+                                time,
+                                FALSE);
 
   return GDK_GRAB_SUCCESS;
 }
@@ -146,19 +120,27 @@ _gdk_windowing_pointer_grab (GdkWindow    *window,
 static void
 break_all_grabs (guint32 time)
 {
-  GdkPointerGrabInfo *grab;
+  GList *list, *l;
+  GdkDeviceManager *device_manager;
 
-  if (_gdk_display->keyboard_grab.window)
-    _gdk_display_unset_has_keyboard_grab (_gdk_display, FALSE);
-
-  grab = _gdk_display_get_last_pointer_grab (_gdk_display);
-  if (grab)
+  device_manager = gdk_display_get_device_manager (_gdk_display);
+  list = gdk_device_manager_list_devices (device_manager,
+                                          GDK_DEVICE_TYPE_MASTER);
+  for (l = list; l; l = l->next)
     {
-      grab->serial_end = 0;
-      grab->implicit_ungrab = TRUE;
+      GdkDeviceGrabInfo *grab;
+
+      grab = _gdk_display_get_last_device_grab (_gdk_display, l->data);
+      if (grab)
+        {
+          grab->serial_end = 0;
+          grab->implicit_ungrab = TRUE;
+        }
+
+      _gdk_display_device_grab_update (_gdk_display, l->data, 0);
     }
 
-  _gdk_display_pointer_grab_update (_gdk_display, 0);
+  g_list_free (list);
 }
 
 static void
@@ -189,19 +171,40 @@ append_event (GdkEvent *event,
 static gint
 gdk_event_apply_filters (NSEvent *nsevent,
 			 GdkEvent *event,
-			 GList *filters)
+			 GList **filters)
 {
   GList *tmp_list;
   GdkFilterReturn result;
   
-  tmp_list = filters;
+  tmp_list = *filters;
 
   while (tmp_list)
     {
       GdkEventFilter *filter = (GdkEventFilter*) tmp_list->data;
-      
-      tmp_list = tmp_list->next;
+      GList *node;
+
+      if ((filter->flags & GDK_EVENT_FILTER_REMOVED) != 0)
+        {
+          tmp_list = tmp_list->next;
+          continue;
+        }
+
+      filter->ref_count++;
       result = filter->function (nsevent, event, filter->data);
+
+      /* get the next node after running the function since the
+         function may add or remove a next node */
+      node = tmp_list;
+      tmp_list = tmp_list->next;
+
+      filter->ref_count--;
+      if (filter->ref_count == 0)
+        {
+          *filters = g_list_remove_link (*filters, node);
+          g_list_free_1 (node);
+          g_free (filter);
+        }
+
       if (result !=  GDK_FILTER_CONTINUE)
 	return result;
     }
@@ -351,11 +354,15 @@ create_focus_event (GdkWindow *window,
 		    gboolean   in)
 {
   GdkEvent *event;
+  GdkDeviceManagerCore *device_manager;
 
   event = gdk_event_new (GDK_FOCUS_CHANGE);
   event->focus_change.window = window;
   event->focus_change.in = in;
 
+  device_manager = GDK_DEVICE_MANAGER_CORE (_gdk_display->device_manager);
+  gdk_event_set_device (event, device_manager->core_keyboard);
+
   return event;
 }
 
@@ -367,7 +374,6 @@ generate_motion_event (GdkWindow *window)
   NSPoint screen_point;
   NSWindow *nswindow;
   GdkQuartzView *view;
-  GdkWindowObject *private;
   GdkEvent *event;
   gint x, y, x_root, y_root;
   GdkDisplay *display;
@@ -376,11 +382,10 @@ generate_motion_event (GdkWindow *window)
   event->any.window = NULL;
   event->any.send_event = TRUE;
 
-  private = (GdkWindowObject *)window;
-  nswindow = ((GdkWindowImplQuartz *)private->impl)->toplevel;
+  nswindow = ((GdkWindowImplQuartz *)window->impl)->toplevel;
   view = (GdkQuartzView *)[nswindow contentView];
 
-  display = gdk_drawable_get_display (window);
+  display = gdk_window_get_display (window);
 
   screen_point = [NSEvent mouseLocation];
 
@@ -389,7 +394,7 @@ generate_motion_event (GdkWindow *window)
   point = [nswindow convertScreenToBase:screen_point];
 
   x = point.x;
-  y = private->height - point.y;
+  y = window->height - point.y;
 
   event->any.type = GDK_MOTION_NOTIFY;
   event->motion.window = window;
@@ -457,7 +462,6 @@ _gdk_quartz_events_send_enter_notify_event (GdkWindow *window)
   NSPoint point;
   NSPoint screen_point;
   NSWindow *nswindow;
-  GdkWindowObject *private;
   GdkEvent *event;
   gint x, y, x_root, y_root;
 
@@ -465,8 +469,7 @@ _gdk_quartz_events_send_enter_notify_event (GdkWindow *window)
   event->any.window = NULL;
   event->any.send_event = FALSE;
 
-  private = (GdkWindowObject *)window;
-  nswindow = ((GdkWindowImplQuartz *)private->impl)->toplevel;
+  nswindow = ((GdkWindowImplQuartz *)window->impl)->toplevel;
 
   screen_point = [NSEvent mouseLocation];
 
@@ -475,7 +478,7 @@ _gdk_quartz_events_send_enter_notify_event (GdkWindow *window)
   point = [nswindow convertScreenToBase:screen_point];
 
   x = point.x;
-  y = private->height - point.y;
+  y = window->height - point.y;
 
   event->crossing.window = window;
   event->crossing.subwindow = NULL;
@@ -488,19 +491,20 @@ _gdk_quartz_events_send_enter_notify_event (GdkWindow *window)
   event->crossing.detail = GDK_NOTIFY_ANCESTOR;
   event->crossing.state = 0;
 
+  gdk_event_set_device (event, _gdk_display->core_pointer);
+
   append_event (event, TRUE);
 }
 
 void
 _gdk_quartz_events_send_map_event (GdkWindow *window)
 {
-  GdkWindowObject *private = (GdkWindowObject *)window;
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (window->impl);
 
   if (!impl->toplevel)
     return;
 
-  if (private->event_mask & GDK_STRUCTURE_MASK)
+  if (window->event_mask & GDK_STRUCTURE_MASK)
     {
       GdkEvent event;
 
@@ -518,26 +522,212 @@ find_toplevel_under_pointer (GdkDisplay *display,
                              gint       *y)
 {
   GdkWindow *toplevel;
+  GdkPointerWindowInfo *info;
 
-  toplevel = display->pointer_info.toplevel_under_pointer;
-  if (toplevel)
+  info = _gdk_display_get_pointer_info (display, display->core_pointer);
+  toplevel = info->toplevel_under_pointer;
+  if (toplevel && WINDOW_IS_TOPLEVEL (toplevel))
     {
-      GdkWindowObject *private;
       NSWindow *nswindow;
       NSPoint point;
 
-      private = (GdkWindowObject *)toplevel;
-      nswindow = ((GdkWindowImplQuartz *)private->impl)->toplevel;
+      nswindow = ((GdkWindowImplQuartz *)toplevel->impl)->toplevel;
 
       point = [nswindow convertScreenToBase:screen_point];
 
       *x = point.x;
-      *y = private->height - point.y;
+      *y = toplevel->height - point.y;
     }
 
   return toplevel;
 }
 
+static GdkWindow *
+find_toplevel_for_keyboard_event (NSEvent *nsevent)
+{
+  GList *list, *l;
+  GdkWindow *window;
+  GdkDisplay *display;
+  GdkQuartzView *view;
+  GdkDeviceManager *device_manager;
+
+  view = (GdkQuartzView *)[[nsevent window] contentView];
+  window = [view gdkWindow];
+
+  display = gdk_window_get_display (window);
+
+  device_manager = gdk_display_get_device_manager (display);
+  list = gdk_device_manager_list_devices (device_manager,
+                                          GDK_DEVICE_TYPE_MASTER);
+  for (l = list; l; l = l->next)
+    {
+      GdkDeviceGrabInfo *grab;
+      GdkDevice *device = l->data;
+
+      if (gdk_device_get_source(device) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      grab = _gdk_display_get_last_device_grab (display, device);
+      if (grab && grab->window && !grab->owner_events)
+        {
+          window = gdk_window_get_effective_toplevel (grab->window);
+          break;
+        }
+    }
+
+  g_list_free (list);
+
+  return window;
+}
+
+static GdkWindow *
+find_toplevel_for_mouse_event (NSEvent    *nsevent,
+                               gint       *x,
+                               gint       *y)
+{
+  NSPoint point;
+  NSPoint screen_point;
+  NSEventType event_type;
+  GdkWindow *toplevel;
+  GdkQuartzView *view;
+  GdkDisplay *display;
+  GdkDeviceGrabInfo *grab;
+
+  view = (GdkQuartzView *)[[nsevent window] contentView];
+  toplevel = [view gdkWindow];
+
+  display = gdk_window_get_display (toplevel);
+
+  event_type = [nsevent type];
+  point = [nsevent locationInWindow];
+  screen_point = [[nsevent window] convertBaseToScreen:point];
+
+  /* From the docs for XGrabPointer:
+   *
+   * If owner_events is True and if a generated pointer event
+   * would normally be reported to this client, it is reported
+   * as usual. Otherwise, the event is reported with respect to
+   * the grab_window and is reported only if selected by
+   * event_mask. For either value of owner_events, unreported
+   * events are discarded.
+   */
+  grab = _gdk_display_get_last_device_grab (display,
+                                            display->core_pointer);
+  if (WINDOW_IS_TOPLEVEL (toplevel) && grab)
+    {
+      /* Implicit grabs do not go through XGrabPointer and thus the
+       * event mask should not be checked.
+       */
+      if (!grab->implicit
+          && (grab->event_mask & get_event_mask_from_ns_event (nsevent)) == 0)
+        return NULL;
+
+      if (grab->owner_events)
+        {
+          /* For owner events, we need to use the toplevel under the
+           * pointer, not the window from the NSEvent, since that is
+           * reported with respect to the key window, which could be
+           * wrong.
+           */
+          GdkWindow *toplevel_under_pointer;
+          gint x_tmp, y_tmp;
+
+          toplevel_under_pointer = find_toplevel_under_pointer (display,
+                                                                screen_point,
+                                                                &x_tmp, &y_tmp);
+          if (toplevel_under_pointer)
+            {
+              toplevel = toplevel_under_pointer;
+              *x = x_tmp;
+              *y = y_tmp;
+            }
+
+          return toplevel;
+        }
+      else
+        {
+          /* Finally check the grab window. */
+          GdkWindow *grab_toplevel;
+          NSWindow *grab_nswindow;
+
+          grab_toplevel = gdk_window_get_effective_toplevel (grab->window);
+
+          grab_nswindow = ((GdkWindowImplQuartz *)grab_toplevel->impl)->toplevel;
+          point = [grab_nswindow convertScreenToBase:screen_point];
+
+          /* Note: x_root and y_root are already right. */
+          *x = point.x;
+          *y = grab_toplevel->height - point.y;
+
+          return grab_toplevel;
+        }
+
+      return NULL;
+    }
+  else 
+    {
+      /* The non-grabbed case. */
+      GdkWindow *toplevel_under_pointer;
+      gint x_tmp, y_tmp;
+
+      /* Ignore all events but mouse moved that might be on the title
+       * bar (above the content view). The reason is that otherwise
+       * gdk gets confused about getting e.g. button presses with no
+       * window (the title bar is not known to it).
+       */
+      if (event_type != NSMouseMoved)
+        if (*y < 0)
+          return NULL;
+
+      /* As for owner events, we need to use the toplevel under the
+       * pointer, not the window from the NSEvent.
+       */
+      toplevel_under_pointer = find_toplevel_under_pointer (display,
+                                                            screen_point,
+                                                            &x_tmp, &y_tmp);
+      if (toplevel_under_pointer
+          && WINDOW_IS_TOPLEVEL (toplevel_under_pointer))
+        {
+          GdkWindowImplQuartz *toplevel_impl;
+
+          toplevel = toplevel_under_pointer;
+
+          toplevel_impl = (GdkWindowImplQuartz *)toplevel->impl;
+
+          if ([toplevel_impl->toplevel showsResizeIndicator])
+            {
+              NSRect frame;
+
+              /* If the resize indicator is visible and the event
+               * is in the lower right 15x15 corner, we leave these
+               * events to Cocoa as to be handled as resize events.
+               * Applications may have widgets in this area.  These
+               * will most likely be larger than 15x15 and for
+               * scroll bars there are also other means to move
+               * the scroll bar.  Since the resize indicator is
+               * the only way of resizing windows on Mac OS, it
+               * is too important to not make functional.
+               */
+              frame = [toplevel_impl->view bounds];
+              if (x_tmp > frame.size.width - GRIP_WIDTH
+                  && x_tmp < frame.size.width
+                  && y_tmp > frame.size.height - GRIP_HEIGHT
+                  && y_tmp < frame.size.height)
+                {
+                  return NULL;
+                }
+            }
+
+          *x = x_tmp;
+          *y = y_tmp;
+        }
+
+      return toplevel;
+    }
+
+  return NULL;
+}
+
 /* This function finds the correct window to send an event to, taking
  * into account grabs, event propagation, and event masks.
  */
@@ -549,22 +739,19 @@ find_window_for_ns_event (NSEvent *nsevent,
                           gint    *y_root)
 {
   GdkQuartzView *view;
-  GdkWindow *toplevel;
-  GdkWindowObject *private;
   NSPoint point;
   NSPoint screen_point;
   NSEventType event_type;
+  GdkWindow *toplevel;
 
   view = (GdkQuartzView *)[[nsevent window] contentView];
-
   toplevel = [view gdkWindow];
-  private = GDK_WINDOW_OBJECT (toplevel);
 
   point = [nsevent locationInWindow];
   screen_point = [[nsevent window] convertBaseToScreen:point];
 
   *x = point.x;
-  *y = private->height - point.y;
+  *y = toplevel->height - point.y;
 
   _gdk_quartz_window_nspoint_to_gdk_xy (screen_point, x_root, y_root);
 
@@ -583,138 +770,7 @@ find_window_for_ns_event (NSEvent *nsevent,
     case NSLeftMouseDragged:
     case NSRightMouseDragged:
     case NSOtherMouseDragged:
-      {
-	GdkDisplay *display;
-        GdkPointerGrabInfo *grab;
-
-        display = gdk_drawable_get_display (toplevel);
-
-	/* From the docs for XGrabPointer:
-	 *
-	 * If owner_events is True and if a generated pointer event
-	 * would normally be reported to this client, it is reported
-	 * as usual. Otherwise, the event is reported with respect to
-	 * the grab_window and is reported only if selected by
-	 * event_mask. For either value of owner_events, unreported
-	 * events are discarded.
-	 */
-        grab = _gdk_display_get_last_pointer_grab (display);
-	if (grab)
-	  {
-            /* Implicit grabs do not go through XGrabPointer and thus the
-             * event mask should not be checked.
-             */
-	    if (!grab->implicit
-                && (grab->event_mask & get_event_mask_from_ns_event (nsevent)) == 0)
-              return NULL;
-
-            if (grab->owner_events)
-              {
-                /* For owner events, we need to use the toplevel under the
-                 * pointer, not the window from the NSEvent, since that is
-                 * reported with respect to the key window, which could be
-                 * wrong.
-                 */
-                GdkWindow *toplevel_under_pointer;
-                gint x_tmp, y_tmp;
-
-                toplevel_under_pointer = find_toplevel_under_pointer (display,
-                                                                      screen_point,
-                                                                      &x_tmp, &y_tmp);
-                if (toplevel_under_pointer)
-                  {
-                    toplevel = toplevel_under_pointer;
-                    *x = x_tmp;
-                    *y = y_tmp;
-                  }
-
-                return toplevel;
-              }
-            else
-              {
-                /* Finally check the grab window. */
-		GdkWindow *grab_toplevel;
-                GdkWindowObject *grab_private;
-                NSWindow *grab_nswindow;
-
-		grab_toplevel = gdk_window_get_toplevel (grab->window);
-                grab_private = (GdkWindowObject *)grab_toplevel;
-
-                grab_nswindow = ((GdkWindowImplQuartz *)grab_private->impl)->toplevel;
-                point = [grab_nswindow convertScreenToBase:screen_point];
-
-                /* Note: x_root and y_root are already right. */
-                *x = point.x;
-                *y = grab_private->height - point.y;
-
-		return grab_toplevel;
-	      }
-
-	    return NULL;
-	  }
-	else 
-	  {
-	    /* The non-grabbed case. */
-            GdkWindow *toplevel_under_pointer;
-            gint x_tmp, y_tmp;
-
-            /* Ignore all events but mouse moved that might be on the title
-             * bar (above the content view). The reason is that otherwise
-             * gdk gets confused about getting e.g. button presses with no
-             * window (the title bar is not known to it).
-             */
-            if (event_type != NSMouseMoved)
-              if (*y < 0)
-                return NULL;
-
-            /* As for owner events, we need to use the toplevel under the
-             * pointer, not the window from the NSEvent.
-             */
-            toplevel_under_pointer = find_toplevel_under_pointer (display,
-                                                                  screen_point,
-                                                                  &x_tmp, &y_tmp);
-            if (toplevel_under_pointer)
-              {
-                GdkWindowObject *toplevel_private;
-                GdkWindowImplQuartz *toplevel_impl;
-
-                toplevel = toplevel_under_pointer;
-
-                toplevel_private = (GdkWindowObject *)toplevel;
-                toplevel_impl = (GdkWindowImplQuartz *)toplevel_private->impl;
-
-                if ([toplevel_impl->toplevel showsResizeIndicator])
-                  {
-                    NSRect frame;
-
-                    /* If the resize indicator is visible and the event
-                     * is in the lower right 15x15 corner, we leave these
-                     * events to Cocoa as to be handled as resize events.
-                     * Applications may have widgets in this area.  These
-                     * will most likely be larger than 15x15 and for
-                     * scroll bars there are also other means to move
-                     * the scroll bar.  Since the resize indicator is
-                     * the only way of resizing windows on Mac OS, it
-                     * is too important to not make functional.
-                     */
-                    frame = [toplevel_impl->view bounds];
-                    if (x_tmp > frame.size.width - GRIP_WIDTH
-                        && x_tmp < frame.size.width
-                        && y_tmp > frame.size.height - GRIP_HEIGHT
-                        && y_tmp < frame.size.height)
-                      {
-                        return NULL;
-                      }
-                  }
-
-                *x = x_tmp;
-                *y = y_tmp;
-              }
-
-            return toplevel;
-	  }
-      }
-      break;
+      return find_toplevel_for_mouse_event (nsevent, x, y);
       
     case NSMouseEntered:
     case NSMouseExited:
@@ -729,10 +785,7 @@ find_window_for_ns_event (NSEvent *nsevent,
     case NSKeyDown:
     case NSKeyUp:
     case NSFlagsChanged:
-      if (_gdk_display->keyboard_grab.window && !_gdk_display->keyboard_grab.owner_events)
-        return gdk_window_get_toplevel (_gdk_display->keyboard_grab.window);
-
-      return toplevel;
+      return find_toplevel_for_keyboard_event (nsevent);
 
     default:
       /* Ignore everything else. */
@@ -766,6 +819,8 @@ fill_crossing_event (GdkWindow       *toplevel,
   event->crossing.detail = detail;
   event->crossing.state = get_keyboard_modifiers_from_ns_event (nsevent);
 
+  gdk_event_set_device (event, _gdk_display->core_pointer);
+
   /* FIXME: Focus and button state? */
 }
 
@@ -864,11 +919,8 @@ fill_scroll_event (GdkWindow          *window,
                    gint                y_root,
                    GdkScrollDirection  direction)
 {
-  GdkWindowObject *private;
   NSPoint point;
 
-  private = GDK_WINDOW_OBJECT (window);
-
   point = [nsevent locationInWindow];
 
   event->any.type = GDK_SCROLL;
@@ -890,6 +942,7 @@ fill_key_event (GdkWindow    *window,
                 GdkEventType  type)
 {
   GdkEventPrivate *priv;
+  GdkDeviceManagerCore *device_manager;
   gchar buf[7];
   gunichar c = 0;
 
@@ -902,7 +955,10 @@ fill_key_event (GdkWindow    *window,
   event->key.state = get_keyboard_modifiers_from_ns_event (nsevent);
   event->key.hardware_keycode = [nsevent keyCode];
   event->key.group = ([nsevent modifierFlags] & NSAlternateKeyMask) ? 1 : 0;
-  event->key.keyval = GDK_VoidSymbol;
+  event->key.keyval = GDK_KEY_VoidSymbol;
+
+  device_manager = GDK_DEVICE_MANAGER_CORE (_gdk_display->device_manager);
+  gdk_event_set_device (event, device_manager->core_keyboard);
   
   gdk_keymap_translate_keyboard_state (NULL,
 				       event->key.hardware_keycode,
@@ -923,23 +979,23 @@ fill_key_event (GdkWindow    *window,
 
       switch (event->key.keyval)
         {
-        case GDK_Meta_R:
-        case GDK_Meta_L:
+        case GDK_KEY_Meta_R:
+        case GDK_KEY_Meta_L:
           mask = GDK_MOD1_MASK;
           break;
-        case GDK_Shift_R:
-        case GDK_Shift_L:
+        case GDK_KEY_Shift_R:
+        case GDK_KEY_Shift_L:
           mask = GDK_SHIFT_MASK;
           break;
-        case GDK_Caps_Lock:
+        case GDK_KEY_Caps_Lock:
           mask = GDK_LOCK_MASK;
           break;
-        case GDK_Alt_R:
-        case GDK_Alt_L:
+        case GDK_KEY_Alt_R:
+        case GDK_KEY_Alt_L:
           mask = GDK_MOD5_MASK;
           break;
-        case GDK_Control_R:
-        case GDK_Control_L:
+        case GDK_KEY_Control_R:
+        case GDK_KEY_Control_L:
           mask = GDK_CONTROL_MASK;
           break;
         default:
@@ -957,7 +1013,7 @@ fill_key_event (GdkWindow    *window,
   event->key.string = NULL;
 
   /* Fill in ->string since apps depend on it, taken from the x11 backend. */
-  if (event->key.keyval != GDK_VoidSymbol)
+  if (event->key.keyval != GDK_KEY_VoidSymbol)
     c = gdk_keyval_to_unicode (event->key.keyval);
 
   if (c)
@@ -974,13 +1030,13 @@ fill_key_event (GdkWindow    *window,
       if (event->key.string)
 	event->key.length = bytes_written;
     }
-  else if (event->key.keyval == GDK_Escape)
+  else if (event->key.keyval == GDK_KEY_Escape)
     {
       event->key.length = 1;
       event->key.string = g_strdup ("\033");
     }
-  else if (event->key.keyval == GDK_Return ||
-	  event->key.keyval == GDK_KP_Enter)
+  else if (event->key.keyval == GDK_KEY_Return ||
+	  event->key.keyval == GDK_KEY_KP_Enter)
     {
       event->key.length = 1;
       event->key.string = g_strdup ("\r");
@@ -1009,17 +1065,13 @@ synthesize_crossing_event (GdkWindow *window,
                            gint       x_root,
                            gint       y_root)
 {
-  GdkWindowObject *private;
-
-  private = GDK_WINDOW_OBJECT (window);
-
   switch ([nsevent type])
     {
     case NSMouseEntered:
       /* Enter events are considered always to be from the root window as we
        * can't know for sure from what window we enter.
        */
-      if (!(private->event_mask & GDK_ENTER_NOTIFY_MASK))
+      if (!(window->event_mask & GDK_ENTER_NOTIFY_MASK))
         return FALSE;
 
       fill_crossing_event (window, event, nsevent,
@@ -1035,7 +1087,7 @@ synthesize_crossing_event (GdkWindow *window,
        * since there is no way to reliably get information about what new
        * window is entered when exiting one.
        */
-      if (!(private->event_mask & GDK_LEAVE_NOTIFY_MASK))
+      if (!(window->event_mask & GDK_LEAVE_NOTIFY_MASK))
         return FALSE;
 
       fill_crossing_event (window, event, nsevent,
@@ -1112,7 +1164,7 @@ gdk_event_translate (GdkEvent *event,
       /* Apply global filters */
       GdkFilterReturn result;
 
-      result = gdk_event_apply_filters (nsevent, event, _gdk_default_filters);
+      result = gdk_event_apply_filters (nsevent, event, &_gdk_default_filters);
       if (result != GDK_FILTER_CONTINUE)
         {
           return_val = (result == GDK_FILTER_TRANSLATE) ? TRUE : FALSE;
@@ -1146,14 +1198,13 @@ gdk_event_translate (GdkEvent *event,
   /* Apply any window filters. */
   if (GDK_IS_WINDOW (window))
     {
-      GdkWindowObject *filter_private = (GdkWindowObject *) window;
       GdkFilterReturn result;
 
-      if (filter_private->filters)
+      if (window->filters)
 	{
 	  g_object_ref (window);
 
-	  result = gdk_event_apply_filters (nsevent, event, filter_private->filters);
+	  result = gdk_event_apply_filters (nsevent, event, &window->filters);
 
 	  g_object_unref (window);
 
@@ -1174,8 +1225,7 @@ gdk_event_translate (GdkEvent *event,
        event_type == NSOtherMouseDown ||
        event_type == NSLeftMouseDown))
     {
-      GdkWindowObject *private = (GdkWindowObject *)window;
-      GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
+      GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (window->impl);
 
       if (![NSApp isActive])
         {
@@ -1184,9 +1234,10 @@ gdk_event_translate (GdkEvent *event,
         }
       else if (![impl->toplevel isKeyWindow])
         {
-          GdkPointerGrabInfo *grab;
+          GdkDeviceGrabInfo *grab;
 
-          grab = _gdk_display_get_last_pointer_grab (_gdk_display);
+          grab = _gdk_display_get_last_device_grab (_gdk_display,
+                                                    _gdk_display->core_pointer);
           if (!grab)
             [impl->toplevel makeKeyWindow];
         }
@@ -1342,14 +1393,6 @@ gdk_display_add_client_message_filter (GdkDisplay   *display,
   /* Not supported. */
 }
 
-void
-gdk_add_client_message_filter (GdkAtom       message_type,
-			       GdkFilterFunc func,
-			       gpointer      data)
-{
-  /* Not supported. */
-}
-
 void
 gdk_display_sync (GdkDisplay *display)
 {
