@@ -1139,8 +1139,8 @@ is_hyper (const gchar *string)
 /**
  * gtk_accelerator_parse:
  * @accelerator:      string representing an accelerator
- * @accelerator_key:  return location for accelerator keyval
- * @accelerator_mods: return location for accelerator modifier mask
+ * @accelerator_key: (out):  return location for accelerator keyval
+ * @accelerator_mods: (out): return location for accelerator modifier mask
  *
  * Parses a string representing an accelerator. The
  * format looks like "&lt;Control&gt;a" or "&lt;Shift&gt;&lt;Alt&gt;F1" or
@@ -1461,7 +1461,7 @@ gtk_accelerator_set_default_mod_mask (GdkModifierType default_mod_mask)
  *
  * Gets the value set by gtk_accelerator_set_default_mod_mask().
  */
-guint
+GdkModifierType
 gtk_accelerator_get_default_mod_mask (void)
 {
   return default_accel_mod_mask;
