@@ -1,7 +1,7 @@
 /* GTK - The GIMP Toolkit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  *
- * GtkBindingSet: Keybinding manager for GtkObjects.
+ * GtkBindingSet: Keybinding manager for GObjects.
  * Copyright (C) 1998 Tim Janik
  *
  * This library is free software; you can redistribute it and/or
@@ -34,6 +34,7 @@
 
 #include "gtkbindings.h"
 #include "gtkkeyhash.h"
+#include "gtktypeutils.h"
 #include "gtkwidget.h"
 #include "gtkrc.h"
 
@@ -46,7 +47,6 @@
 typedef struct {
   GtkPathType   type;
   GPatternSpec *pspec;
-  GSList       *path;
   gpointer      user_data;
   guint         seq_id;
 } PatternSpec;
@@ -64,7 +64,6 @@ static GQuark		 key_id_class_binding_set = 0;
 static void
 pattern_spec_free (PatternSpec *pspec)
 {
-  _gtk_rc_free_widget_class_path (pspec->path);
   if (pspec->pspec)
     g_pattern_spec_free (pspec->pspec);
   g_free (pspec);
@@ -131,8 +130,8 @@ binding_key_hash_insert_entry (GtkKeyHash      *key_hash,
    */
   if (entry->modifiers & GDK_SHIFT_MASK)
     {
-      if (keyval == GDK_Tab)
-	keyval = GDK_ISO_Left_Tab;
+      if (keyval == GDK_KEY_Tab)
+	keyval = GDK_KEY_ISO_Left_Tab;
       else
 	keyval = gdk_keyval_to_upper (keyval);
     }
@@ -340,7 +339,7 @@ binding_ht_lookup_entry (GtkBindingSet  *set,
 }
 
 static gboolean
-binding_compose_params (GtkObject       *object,
+binding_compose_params (GObject         *object,
 			GtkBindingArg	*args,
 			GSignalQuery	*query,
 			GValue	       **params_p)
@@ -469,7 +468,7 @@ binding_compose_params (GtkObject       *object,
 
 static gboolean
 gtk_binding_entry_activate (GtkBindingEntry *entry,
-			    GtkObject	    *object)
+			    GObject	    *object)
 {
   GtkBindingSignal *sig;
   gboolean old_emission;
@@ -593,7 +592,7 @@ gtk_binding_set_new (const gchar *set_name)
 
 /**
  * gtk_binding_set_by_class:
- * @object_class: a valid #GtkObject class
+ * @object_class: a valid #GObject class
  *
  * This function returns the binding set named after the type name of
  * the passed in class structure. New binding sets are created on
@@ -604,10 +603,10 @@ gtk_binding_set_new (const gchar *set_name)
 GtkBindingSet*
 gtk_binding_set_by_class (gpointer object_class)
 {
-  GtkObjectClass *class = object_class;
+  GObjectClass *class = object_class;
   GtkBindingSet* binding_set;
 
-  g_return_val_if_fail (GTK_IS_OBJECT_CLASS (class), NULL);
+  g_return_val_if_fail (G_IS_OBJECT_CLASS (class), NULL);
 
   if (!key_id_class_binding_set)
     key_id_class_binding_set = g_quark_from_static_string (key_class_binding_set);
@@ -671,12 +670,12 @@ gboolean
 gtk_binding_set_activate (GtkBindingSet	 *binding_set,
 			  guint		  keyval,
 			  GdkModifierType modifiers,
-			  GtkObject	 *object)
+			  GObject	 *object)
 {
   GtkBindingEntry *entry;
   
   g_return_val_if_fail (binding_set != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_OBJECT (object), FALSE);
+  g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   
   keyval = gdk_keyval_to_lower (keyval);
   modifiers = modifiers & BINDING_MOD_MASK ();
@@ -769,7 +768,8 @@ gtk_binding_entry_remove (GtkBindingSet	 *binding_set,
  * @keyval:       key value
  * @modifiers:    key modifier
  * @signal_name:  signal name to be bound
- * @binding_args: list of #GtkBindingArg signal arguments
+ * @binding_args: (transfer none) (element-type GtkBindingArg):
+ *     list of #GtkBindingArg signal arguments
  *
  * Override or install a new key binding for @keyval with @modifiers on
  * @binding_set.
@@ -970,6 +970,8 @@ gtk_binding_entry_add_signal (GtkBindingSet  *binding_set,
  *
  * This function is used internally by the GtkRC parsing mechanism to
  * assign match patterns to #GtkBindingSet structures.
+ *
+ * Deprecated: 3.0
  */
 void
 gtk_binding_set_add_path (GtkBindingSet	     *binding_set,
@@ -1007,16 +1009,10 @@ gtk_binding_set_add_path (GtkBindingSet	     *binding_set,
   pspec = g_new (PatternSpec, 1);
   pspec->type = path_type;
   if (path_type == GTK_PATH_WIDGET_CLASS)
-    {
-      pspec->pspec = NULL;
-      pspec->path = _gtk_rc_parse_widget_class_path (path_pattern);
-    }
+    pspec->pspec = NULL;
   else
-    {
-      pspec->pspec = g_pattern_spec_new (path_pattern);
-      pspec->path = NULL;
-    }
-    
+    pspec->pspec = g_pattern_spec_new (path_pattern);
+
   pspec->seq_id = priority << 28;
   pspec->user_data = binding_set;
   
@@ -1051,7 +1047,7 @@ gtk_binding_set_add_path (GtkBindingSet	     *binding_set,
 
 static gboolean
 binding_match_activate (GSList          *pspec_list,
-			GtkObject	*object,
+			GObject	        *object,
 			guint	         path_length,
 			gchar           *path,
 			gchar           *path_reversed,
@@ -1068,13 +1064,8 @@ binding_match_activate (GSList          *pspec_list,
 
       binding_set = NULL;
       pspec = slist->data;
-      
-      if (pspec->type == GTK_PATH_WIDGET_CLASS)
-        {
-          if (_gtk_rc_match_widget_class (pspec->path, path_length, path, path_reversed))
-	    binding_set = pspec->user_data;
-        }
-      else
+
+      if (pspec->type != GTK_PATH_WIDGET_CLASS)
         {
           if (g_pattern_match (pspec->pspec, path_length, path, path_reversed))
 	    binding_set = pspec->user_data;
@@ -1169,51 +1160,16 @@ gtk_binding_entries_sort_patterns (GSList      *entries,
 }
 
 static gboolean
-gtk_bindings_activate_list (GtkObject *object,
+gtk_bindings_activate_list (GObject   *object,
 			    GSList    *entries,
 			    gboolean   is_release)
 {
-  GtkWidget *widget = GTK_WIDGET (object);
   gboolean handled = FALSE;
 
   if (!entries)
     return FALSE;
 
-  if (!handled)
-    {
-      guint path_length;
-      gchar *path, *path_reversed;
-      GSList *patterns;
-      gboolean unbound;
-
-      gtk_widget_path (widget, &path_length, &path, &path_reversed);
-      patterns = gtk_binding_entries_sort_patterns (entries, GTK_PATH_WIDGET, is_release);
-      handled = binding_match_activate (patterns, object, path_length, path, path_reversed, &unbound);
-      g_slist_free (patterns);
-      g_free (path);
-      g_free (path_reversed);
-
-      if (unbound)
-        return FALSE;
-    }
-
-  if (!handled)
-    {
-      guint path_length;
-      gchar *path, *path_reversed;
-      GSList *patterns;
-      gboolean unbound;
-
-      gtk_widget_class_path (widget, &path_length, &path, &path_reversed);
-      patterns = gtk_binding_entries_sort_patterns (entries, GTK_PATH_WIDGET_CLASS, is_release);
-      handled = binding_match_activate (patterns, object, path_length, path, path_reversed, &unbound);
-      g_slist_free (patterns);
-      g_free (path);
-      g_free (path_reversed);
-
-      if (unbound)
-        return FALSE;
-    }
+  /* FIXME: Add back binding parsing from user config files */
 
   if (!handled)
     {
@@ -1263,7 +1219,7 @@ gtk_bindings_activate_list (GtkObject *object,
  * Return value: %TRUE if a binding was found and activated
  */
 gboolean
-gtk_bindings_activate (GtkObject       *object,
+gtk_bindings_activate (GObject         *object,
 		       guint	        keyval,
 		       GdkModifierType  modifiers)
 {
@@ -1273,8 +1229,6 @@ gtk_bindings_activate (GtkObject       *object,
   gboolean handled = FALSE;
   gboolean is_release;
 
-  g_return_val_if_fail (GTK_IS_OBJECT (object), FALSE);
-
   if (!GTK_IS_WIDGET (object))
     return FALSE;
 
@@ -1295,7 +1249,7 @@ gtk_bindings_activate (GtkObject       *object,
 
 /**
  * gtk_bindings_activate_event:
- * @object: a #GtkObject (generally must be a widget)
+ * @object: a #GObject (generally must be a widget)
  * @event: a #GdkEventKey
  * 
  * Looks up key bindings for @object to find one matching
@@ -1306,7 +1260,7 @@ gtk_bindings_activate (GtkObject       *object,
  * Since: 2.4
  */
 gboolean
-gtk_bindings_activate_event (GtkObject   *object,
+gtk_bindings_activate_event (GObject     *object,
                              GdkEventKey *event)
 {
   GSList *entries = NULL;
@@ -1314,8 +1268,6 @@ gtk_bindings_activate_event (GtkObject   *object,
   GtkKeyHash *key_hash;
   gboolean handled = FALSE;
 
-  g_return_val_if_fail (GTK_IS_OBJECT (object), FALSE);
-
   if (!GTK_IS_WIDGET (object))
     return FALSE;
 
@@ -1335,343 +1287,3 @@ gtk_bindings_activate_event (GtkObject   *object,
 
   return handled;
 }
-
-static guint
-gtk_binding_parse_signal (GScanner       *scanner,
-			  GtkBindingSet  *binding_set,
-			  guint		  keyval,
-			  GdkModifierType modifiers)
-{
-  gchar *signal;
-  guint expected_token = 0;
-  GSList *args;
-  GSList *slist;
-  gboolean done;
-  gboolean negate;
-  gboolean need_arg;
-  gboolean seen_comma;
-
-  g_return_val_if_fail (scanner != NULL, G_TOKEN_ERROR);
-  
-  g_scanner_get_next_token (scanner);
-  if (scanner->token != G_TOKEN_STRING)
-    return G_TOKEN_STRING;
-  g_scanner_peek_next_token (scanner);
-  if (scanner->next_token != '(')
-    {
-      g_scanner_get_next_token (scanner);
-      return '(';
-    }
-  signal = g_strdup (scanner->value.v_string);
-  g_scanner_get_next_token (scanner);
-
-  negate = FALSE;
-  args = NULL;
-  done = FALSE;
-  need_arg = TRUE;
-  seen_comma = FALSE;
-  scanner->config->scan_symbols = FALSE;
-  do
-    {
-      if (need_arg)
-	expected_token = G_TOKEN_INT;
-      else
-	expected_token = ')';
-      g_scanner_get_next_token (scanner);
-      switch (scanner->token)
-	{
-	  GtkBindingArg *arg;
-
-	case G_TOKEN_FLOAT:
-	  if (need_arg)
-	    {
-	      need_arg = FALSE;
-	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = G_TYPE_DOUBLE;
-	      arg->d.double_data = scanner->value.v_float;
-	      if (negate)
-		{
-		  arg->d.double_data = - arg->d.double_data;
-		  negate = FALSE;
-		}
-	      args = g_slist_prepend (args, arg);
-	    }
-	  else
-	    done = TRUE;
-	  break;
-	case G_TOKEN_INT:
-	  if (need_arg)
-	    {
-	      need_arg = FALSE;
-	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = G_TYPE_LONG;
-	      arg->d.long_data = scanner->value.v_int;
-	      if (negate)
-		{
-		  arg->d.long_data = - arg->d.long_data;
-		  negate = FALSE;
-		}
-	      args = g_slist_prepend (args, arg);
-	    }
-          else
-	    done = TRUE;
-	  break;
-	case G_TOKEN_STRING:
-	  if (need_arg && !negate)
-	    {
-	      need_arg = FALSE;
-	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = G_TYPE_STRING;
-	      arg->d.string_data = g_strdup (scanner->value.v_string);
-	      args = g_slist_prepend (args, arg);
-	    }
-	  else
-	    done = TRUE;
-	  break;
-	case G_TOKEN_IDENTIFIER:
-	  if (need_arg && !negate)
-	    {
-	      need_arg = FALSE;
-	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = GTK_TYPE_IDENTIFIER;
-	      arg->d.string_data = g_strdup (scanner->value.v_identifier);
-	      args = g_slist_prepend (args, arg);
-	    }
-	  else
-	    done = TRUE;
-	  break;
-	case '-':
-	  if (!need_arg)
-	    done = TRUE;
-	  else if (negate)
-	    {
-	      expected_token = G_TOKEN_INT;
-	      done = TRUE;
-	    }
-	  else
-	    negate = TRUE;
-	  break;
-	case ',':
-	  seen_comma = TRUE;
-	  if (need_arg)
-	    done = TRUE;
-	  else
-	    need_arg = TRUE;
-	  break;
-	case ')':
-	  if (!(need_arg && seen_comma) && !negate)
-	    {
-	      args = g_slist_reverse (args);
-	      _gtk_binding_entry_add_signall (binding_set,
-                                              keyval,
-                                              modifiers,
-                                              signal,
-                                              args);
-	      expected_token = G_TOKEN_NONE;
-	    }
-	  done = TRUE;
-	  break;
-	default:
-	  done = TRUE;
-	  break;
-	}
-    }
-  while (!done);
-  scanner->config->scan_symbols = TRUE;
-  
-  for (slist = args; slist; slist = slist->next)
-    {
-      GtkBindingArg *arg;
-
-      arg = slist->data;
-      if (G_TYPE_FUNDAMENTAL (arg->arg_type) == G_TYPE_STRING)
-	g_free (arg->d.string_data);
-      g_free (arg);
-    }
-  g_slist_free (args);
-  g_free (signal);
-
-  return expected_token;
-}
-
-static inline guint
-gtk_binding_parse_bind (GScanner       *scanner,
-			GtkBindingSet  *binding_set)
-{
-  guint keyval = 0;
-  GdkModifierType modifiers = 0;
-  gboolean unbind = FALSE;
-
-  g_return_val_if_fail (scanner != NULL, G_TOKEN_ERROR);
-  
-  g_scanner_get_next_token (scanner);
-  if (scanner->token != GTK_RC_TOKEN_BIND &&
-      scanner->token != GTK_RC_TOKEN_UNBIND)
-    return GTK_RC_TOKEN_BIND;
-  unbind = scanner->token == GTK_RC_TOKEN_UNBIND;
-  g_scanner_get_next_token (scanner);
-  if (scanner->token != G_TOKEN_STRING)
-    return G_TOKEN_STRING;
-  gtk_accelerator_parse (scanner->value.v_string, &keyval, &modifiers);
-  modifiers &= BINDING_MOD_MASK ();
-  if (keyval == 0)
-    return G_TOKEN_STRING;
-
-  if (unbind)
-    {
-      gtk_binding_entry_skip (binding_set, keyval, modifiers);
-      return G_TOKEN_NONE;
-    }
-
-  g_scanner_get_next_token (scanner);
-
-  if (scanner->token != '{')
-    return '{';
-
-  gtk_binding_entry_clear_internal (binding_set, keyval, modifiers);
-
-  g_scanner_peek_next_token (scanner);
-  while (scanner->next_token != '}')
-    {
-      switch (scanner->next_token)
-	{
-	  guint expected_token;
-
-	case G_TOKEN_STRING:
-	  expected_token = gtk_binding_parse_signal (scanner,
-						     binding_set,
-						     keyval,
-						     modifiers);
-	  if (expected_token != G_TOKEN_NONE)
-	    return expected_token;
-	  break;
-	default:
-	  g_scanner_get_next_token (scanner);
-	  return '}';
-	}
-      g_scanner_peek_next_token (scanner);
-    }
-  g_scanner_get_next_token (scanner);
-
-  return G_TOKEN_NONE;
-}
-
-guint
-_gtk_binding_parse_binding (GScanner *scanner)
-{
-  gchar *name;
-  GtkBindingSet *binding_set;
-
-  g_return_val_if_fail (scanner != NULL, G_TOKEN_ERROR);
-
-  g_scanner_get_next_token (scanner);
-  if (scanner->token != GTK_RC_TOKEN_BINDING)
-    return GTK_RC_TOKEN_BINDING;
-  g_scanner_get_next_token (scanner);
-  if (scanner->token != G_TOKEN_STRING)
-    return G_TOKEN_STRING;
-  name = g_strdup (scanner->value.v_string);
-
-  g_scanner_get_next_token (scanner);
-  if (scanner->token != '{')
-    {
-      g_free (name);
-      return G_TOKEN_STRING;
-    }
-
-  binding_set = gtk_binding_set_find (name);
-  if (!binding_set)
-    {
-      binding_set = gtk_binding_set_new (name);
-      binding_set->parsed = 1;
-    }
-  g_free (name);
-
-  g_scanner_peek_next_token (scanner);
-  while (scanner->next_token != '}')
-    {
-      switch (scanner->next_token)
-	{
-	  guint expected_token;
-
-	case GTK_RC_TOKEN_BIND:
-	case GTK_RC_TOKEN_UNBIND:
-	  expected_token = gtk_binding_parse_bind (scanner, binding_set);
-	  if (expected_token != G_TOKEN_NONE)
-	    return expected_token;
-	  break;
-	default:
-	  g_scanner_get_next_token (scanner);
-	  return '}';
-	}
-      g_scanner_peek_next_token (scanner);
-    }
-  g_scanner_get_next_token (scanner);
-
-  return G_TOKEN_NONE;
-}
-
-static void
-free_pattern_specs (GSList *pattern_specs)
-{
-  GSList *slist;
-
-  for (slist = pattern_specs; slist; slist = slist->next)
-    {
-      PatternSpec *pspec;
-
-      pspec = slist->data;
-
-      pattern_spec_free (pspec);
-    }
-
-  g_slist_free (pattern_specs);
-}
-
-static void
-binding_set_delete (GtkBindingSet *binding_set)
-{
-  GtkBindingEntry *entry, *next;
-
-  entry = binding_set->entries;
-  while (entry)
-    {
-      next = entry->set_next;
-      binding_entry_destroy (entry);
-      entry = next;
-    }
-  
-  free_pattern_specs (binding_set->widget_path_pspecs);
-  free_pattern_specs (binding_set->widget_class_pspecs);
-  free_pattern_specs (binding_set->class_branch_pspecs);
-
-  g_free (binding_set);
-}
-
-/**
- * _gtk_binding_reset_parsed:
- * 
- * Remove all binding sets that were added by gtk_binding_parse_binding().
- */
-void
-_gtk_binding_reset_parsed (void)
-{
-  GSList *slist, *next;
-  
-  slist = binding_set_list;
-  while (slist)
-    {
-      GtkBindingSet *binding_set;
-
-      binding_set = slist->data;
-      next = slist->next;
-
-      if (binding_set->parsed)
-	{
-	  binding_set_list = g_slist_delete_link (binding_set_list, slist);
-	  binding_set_delete (binding_set);
-	}
-
-      slist = next;
-    }
-}
