@@ -31,12 +31,12 @@
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkbuildable.h"
-#include "gtksizerequest.h"
+
 
 #define LABEL_PAD 1
 #define LABEL_SIDE_PAD 2
 
-struct _GtkFramePriv
+struct _GtkFramePrivate
 {
   /* Properties */
   GtkWidget *label_widget;
@@ -55,7 +55,6 @@ enum {
   PROP_LABEL,
   PROP_LABEL_XALIGN,
   PROP_LABEL_YALIGN,
-  PROP_SHADOW,
   PROP_SHADOW_TYPE,
   PROP_LABEL_WIDGET
 };
@@ -68,10 +67,8 @@ static void gtk_frame_get_property (GObject     *object,
 				    guint        param_id,
 				    GValue      *value,
 				    GParamSpec  *pspec);
-static void gtk_frame_paint         (GtkWidget      *widget,
-				     GdkRectangle   *area);
-static gint gtk_frame_expose        (GtkWidget      *widget,
-				     GdkEventExpose *event);
+static gboolean gtk_frame_draw      (GtkWidget      *widget,
+				     cairo_t        *cr);
 static void gtk_frame_size_allocate (GtkWidget      *widget,
 				     GtkAllocation  *allocation);
 static void gtk_frame_remove        (GtkContainer   *container,
@@ -93,18 +90,17 @@ static void gtk_frame_buildable_add_child           (GtkBuildable *buildable,
 						     GObject      *child,
 						     const gchar  *type);
 
-static void gtk_frame_size_request_init             (GtkSizeRequestIface *iface);
-static void gtk_frame_get_width                     (GtkSizeRequest      *widget,
+static void gtk_frame_get_preferred_width           (GtkWidget           *widget,
                                                      gint                *minimum_size,
 						     gint                *natural_size);
-static void gtk_frame_get_height                    (GtkSizeRequest      *widget,
+static void gtk_frame_get_preferred_height          (GtkWidget           *widget,
 						     gint                *minimum_size,
 						     gint                *natural_size);
-static void gtk_frame_get_height_for_width          (GtkSizeRequest      *layout,
+static void gtk_frame_get_preferred_height_for_width(GtkWidget           *layout,
 						     gint                 width,
 						     gint                *minimum_height,
 						     gint                *natural_height);
-static void gtk_frame_get_width_for_height          (GtkSizeRequest      *layout,
+static void gtk_frame_get_preferred_width_for_height(GtkWidget           *layout,
 						     gint                 width,
 						     gint                *minimum_height,
 						     gint                *natural_height);
@@ -112,9 +108,7 @@ static void gtk_frame_get_width_for_height          (GtkSizeRequest      *layout
 
 G_DEFINE_TYPE_WITH_CODE (GtkFrame, gtk_frame, GTK_TYPE_BIN,
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
-						gtk_frame_buildable_init)
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SIZE_REQUEST,
-                                                gtk_frame_size_request_init))
+						gtk_frame_buildable_init))
 
 static void
 gtk_frame_class_init (GtkFrameClass *class)
@@ -156,13 +150,6 @@ gtk_frame_class_init (GtkFrameClass *class)
 						       1.0,
 						       0.5,
 						       GTK_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_SHADOW,
-                                   g_param_spec_enum ("shadow", NULL,
-                                                      P_("Deprecated property, use shadow_type instead"),
-						      GTK_TYPE_SHADOW_TYPE,
-						      GTK_SHADOW_ETCHED_IN,
-                                                      GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW_TYPE,
                                    g_param_spec_enum ("shadow-type",
@@ -180,15 +167,19 @@ gtk_frame_class_init (GtkFrameClass *class)
                                                         GTK_TYPE_WIDGET,
                                                         GTK_PARAM_READWRITE));
 
-  widget_class->expose_event = gtk_frame_expose;
-  widget_class->size_allocate = gtk_frame_size_allocate;
+  widget_class->draw                           = gtk_frame_draw;
+  widget_class->size_allocate                  = gtk_frame_size_allocate;
+  widget_class->get_preferred_width            = gtk_frame_get_preferred_width;
+  widget_class->get_preferred_height           = gtk_frame_get_preferred_height;
+  widget_class->get_preferred_height_for_width = gtk_frame_get_preferred_height_for_width;
+  widget_class->get_preferred_width_for_height = gtk_frame_get_preferred_width_for_height;
 
   container_class->remove = gtk_frame_remove;
   container_class->forall = gtk_frame_forall;
 
   class->compute_child_allocation = gtk_frame_real_compute_child_allocation;
 
-  g_type_class_add_private (class, sizeof (GtkFramePriv));
+  g_type_class_add_private (class, sizeof (GtkFramePrivate));
 }
 
 static void
@@ -214,11 +205,11 @@ gtk_frame_buildable_add_child (GtkBuildable *buildable,
 static void
 gtk_frame_init (GtkFrame *frame)
 {
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
 
   frame->priv = G_TYPE_INSTANCE_GET_PRIVATE (frame,
                                              GTK_TYPE_FRAME,
-                                             GtkFramePriv);
+                                             GtkFramePrivate);
   priv = frame->priv;
 
   priv->label_widget = NULL;
@@ -234,7 +225,7 @@ gtk_frame_set_property (GObject         *object,
 			GParamSpec      *pspec)
 {
   GtkFrame *frame = GTK_FRAME (object);
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
 
   switch (prop_id)
     {
@@ -249,7 +240,6 @@ gtk_frame_set_property (GObject         *object,
       gtk_frame_set_label_align (frame, priv->label_xalign,
 				 g_value_get_float (value));
       break;
-    case PROP_SHADOW:
     case PROP_SHADOW_TYPE:
       gtk_frame_set_shadow_type (frame, g_value_get_enum (value));
       break;
@@ -269,7 +259,7 @@ gtk_frame_get_property (GObject         *object,
 			GParamSpec      *pspec)
 {
   GtkFrame *frame = GTK_FRAME (object);
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
 
   switch (prop_id)
     {
@@ -282,7 +272,6 @@ gtk_frame_get_property (GObject         *object,
     case PROP_LABEL_YALIGN:
       g_value_set_float (value, priv->label_yalign);
       break;
-    case PROP_SHADOW:
     case PROP_SHADOW_TYPE:
       g_value_set_enum (value, priv->shadow_type);
       break;
@@ -317,7 +306,7 @@ gtk_frame_remove (GtkContainer *container,
 		  GtkWidget    *child)
 {
   GtkFrame *frame = GTK_FRAME (container);
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
 
   if (priv->label_widget == child)
     gtk_frame_set_label_widget (frame, NULL);
@@ -333,7 +322,7 @@ gtk_frame_forall (GtkContainer *container,
 {
   GtkBin *bin = GTK_BIN (container);
   GtkFrame *frame = GTK_FRAME (container);
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
   GtkWidget *child;
 
   child = gtk_bin_get_child (bin);
@@ -388,7 +377,7 @@ gtk_frame_set_label (GtkFrame *frame,
 G_CONST_RETURN gchar *
 gtk_frame_get_label (GtkFrame *frame)
 {
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
 
   g_return_val_if_fail (GTK_IS_FRAME (frame), NULL);
 
@@ -413,12 +402,12 @@ void
 gtk_frame_set_label_widget (GtkFrame  *frame,
 			    GtkWidget *label_widget)
 {
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
   gboolean need_resize = FALSE;
 
   g_return_if_fail (GTK_IS_FRAME (frame));
   g_return_if_fail (label_widget == NULL || GTK_IS_WIDGET (label_widget));
-  g_return_if_fail (label_widget == NULL || label_widget->parent == NULL);
+  g_return_if_fail (label_widget == NULL || gtk_widget_get_parent (label_widget) == NULL);
 
   priv = frame->priv;
 
@@ -456,7 +445,7 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
  * Retrieves the label widget for the frame. See
  * gtk_frame_set_label_widget().
  *
- * Return value: the label widget, or %NULL if there is none.
+ * Return value: (transfer none): the label widget, or %NULL if there is none.
  **/
 GtkWidget *
 gtk_frame_get_label_widget (GtkFrame *frame)
@@ -485,7 +474,7 @@ gtk_frame_set_label_align (GtkFrame *frame,
 			   gfloat    xalign,
 			   gfloat    yalign)
 {
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
 
   g_return_if_fail (GTK_IS_FRAME (frame));
 
@@ -525,7 +514,7 @@ gtk_frame_get_label_align (GtkFrame *frame,
 		           gfloat   *xalign,
 			   gfloat   *yalign)
 {
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
 
   g_return_if_fail (GTK_IS_FRAME (frame));
 
@@ -548,7 +537,7 @@ void
 gtk_frame_set_shadow_type (GtkFrame      *frame,
 			   GtkShadowType  type)
 {
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
   GtkWidget *widget;
 
   g_return_if_fail (GTK_IS_FRAME (frame));
@@ -587,74 +576,67 @@ gtk_frame_get_shadow_type (GtkFrame *frame)
   return frame->priv->shadow_type;
 }
 
-static void
-gtk_frame_paint (GtkWidget    *widget,
-		 GdkRectangle *area)
+static gboolean
+gtk_frame_draw (GtkWidget *widget,
+		cairo_t   *cr)
 {
   GtkFrame *frame;
-  GtkFramePriv *priv;
+  GtkFramePrivate *priv;
+  GtkStateType state;
+  GtkStyle *style;
   gint x, y, width, height;
+  GtkAllocation allocation;
 
-  if (gtk_widget_is_drawable (widget))
+  frame = GTK_FRAME (widget);
+  priv = frame->priv;
+
+  style = gtk_widget_get_style (widget);
+  state = gtk_widget_get_state (widget);
+  gtk_widget_get_allocation (widget, &allocation);
+
+  x = priv->child_allocation.x - allocation.x - style->xthickness;
+  y = priv->child_allocation.y - allocation.y - style->ythickness;
+  width = priv->child_allocation.width + 2 * style->xthickness;
+  height =  priv->child_allocation.height + 2 * style->ythickness;
+
+  if (priv->label_widget)
     {
-      frame = GTK_FRAME (widget);
-      priv = frame->priv;
+      gfloat xalign;
+      gint height_extra;
+      gint x2;
 
-      x = priv->child_allocation.x - widget->style->xthickness;
-      y = priv->child_allocation.y - widget->style->ythickness;
-      width = priv->child_allocation.width + 2 * widget->style->xthickness;
-      height =  priv->child_allocation.height + 2 * widget->style->ythickness;
+      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
+        xalign = priv->label_xalign;
+      else
+        xalign = 1 - priv->label_xalign;
 
-      if (priv->label_widget)
-	{
-	  gfloat xalign;
-	  gint height_extra;
-	  gint x2;
+      height_extra = MAX (0, priv->label_allocation.height - style->ythickness)
+                     - priv->label_yalign * priv->label_allocation.height;
+      y -= height_extra;
+      height += height_extra;
 
-	  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
-	    xalign = priv->label_xalign;
-	  else
-	    xalign = 1 - priv->label_xalign;
-
-	  height_extra = MAX (0, priv->label_allocation.height - widget->style->ythickness)
-	    - priv->label_yalign * priv->label_allocation.height;
-	  y -= height_extra;
-	  height += height_extra;
-	  
-	  x2 = widget->style->xthickness + (priv->child_allocation.width - priv->label_allocation.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD) * xalign + LABEL_SIDE_PAD;
-	  
-	  /* If the label is completely over or under the frame we can omit the gap */
-	  if (priv->label_yalign == 0.0 || priv->label_yalign == 1.0)
-	    gtk_paint_shadow (widget->style, widget->window,
-			      widget->state, priv->shadow_type,
-			      area, widget, "frame",
-			      x, y, width, height);
-	  else
-	    gtk_paint_shadow_gap (widget->style, widget->window,
-				  widget->state, priv->shadow_type,
-				  area, widget, "frame",
-				  x, y, width, height,
-				  GTK_POS_TOP,
-				  x2, priv->label_allocation.width + 2 * LABEL_PAD);
-	}
-       else
-	 gtk_paint_shadow (widget->style, widget->window,
-			   widget->state, priv->shadow_type,
-			   area, widget, "frame",
-			   x, y, width, height);
+      x2 = style->xthickness + (priv->child_allocation.width - priv->label_allocation.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD) * xalign + LABEL_SIDE_PAD;
+      /* If the label is completely over or under the frame we can omit the gap */
+      if (priv->label_yalign == 0.0 || priv->label_yalign == 1.0)
+        gtk_paint_shadow (style, cr,
+                          state, priv->shadow_type,
+                          widget, "frame",
+                          x, y, width, height);
+      else
+        gtk_paint_shadow_gap (style, cr,
+                              state, priv->shadow_type,
+                              widget, "frame",
+                              x, y, width, height,
+                              GTK_POS_TOP,
+                              x2, priv->label_allocation.width + 2 * LABEL_PAD);
     }
-}
+   else
+     gtk_paint_shadow (style, cr,
+                       state, priv->shadow_type,
+                       widget, "frame",
+                       x, y, width, height);
 
-static gboolean
-gtk_frame_expose (GtkWidget      *widget,
-		  GdkEventExpose *event)
-{
-  if (gtk_widget_is_drawable (widget))
-    {
-      gtk_frame_paint (widget, &event->area);
-
-      GTK_WIDGET_CLASS (gtk_frame_parent_class)->expose_event (widget, event);
-    }
+  GTK_WIDGET_CLASS (gtk_frame_parent_class)->draw (widget, cr);
 
   return FALSE;
 }
@@ -664,28 +646,22 @@ gtk_frame_size_allocate (GtkWidget     *widget,
 			 GtkAllocation *allocation)
 {
   GtkFrame *frame = GTK_FRAME (widget);
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
   GtkBin *bin = GTK_BIN (widget);
   GtkAllocation new_allocation;
   GtkWidget *child;
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
   gtk_frame_compute_child_allocation (frame, &new_allocation);
   
   /* If the child allocation changed, that means that the frame is drawn
    * in a new place, so we must redraw the entire widget.
    */
-  if (gtk_widget_get_mapped (widget)
-#if 0
-      &&
-      (new_allocation.x != priv->child_allocation.x ||
-       new_allocation.y != priv->child_allocation.y ||
-       new_allocation.width != priv->child_allocation.width ||
-       new_allocation.height != priv->child_allocation.height)
-#endif
-     )
-    gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
+  if (gtk_widget_get_mapped (widget))
+    {
+      gdk_window_invalidate_rect (gtk_widget_get_window (widget), allocation, FALSE);
+    }
 
   child = gtk_bin_get_child (bin);
   if (child && gtk_widget_get_visible (child))
@@ -695,20 +671,23 @@ gtk_frame_size_allocate (GtkWidget     *widget,
 
   if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
     {
+      GtkStyle *style;
       gint nat_width, width, height;
       gfloat xalign;
 
+      style = gtk_widget_get_style (widget);
+
       if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
 	xalign = priv->label_xalign;
       else
 	xalign = 1 - priv->label_xalign;
 
-      gtk_size_request_get_width (GTK_SIZE_REQUEST (priv->label_widget), NULL, &nat_width);
+      gtk_widget_get_preferred_width (priv->label_widget, NULL, &nat_width);
       width = new_allocation.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD;
       width = MIN (width, nat_width);
 
-      gtk_size_request_get_height_for_width (GTK_SIZE_REQUEST (priv->label_widget), width,
-					     &height, NULL);
+      gtk_widget_get_preferred_height_for_width (priv->label_widget, width,
+                                                 &height, NULL);
 
 
       priv->label_allocation.x = priv->child_allocation.x + LABEL_SIDE_PAD +
@@ -716,8 +695,7 @@ gtk_frame_size_allocate (GtkWidget     *widget,
 
       priv->label_allocation.width = width;
 
-
-      priv->label_allocation.y = priv->child_allocation.y - MAX (height, widget->style->ythickness);
+      priv->label_allocation.y = priv->child_allocation.y - MAX (height, style->ythickness);
       priv->label_allocation.height = height;
 
       gtk_widget_size_allocate (priv->label_widget, &priv->label_allocation);
@@ -738,76 +716,82 @@ static void
 gtk_frame_real_compute_child_allocation (GtkFrame      *frame,
 					 GtkAllocation *child_allocation)
 {
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
   GtkWidget *widget = GTK_WIDGET (frame);
-  GtkAllocation *allocation = &widget->allocation;
+  GtkAllocation allocation;
+  GtkStyle *style;
   gint top_margin;
   guint border_width;
 
+  style = gtk_widget_get_style (widget);
+
+  gtk_widget_get_allocation (widget, &allocation);
+
   border_width = gtk_container_get_border_width (GTK_CONTAINER (frame));
 
   if (priv->label_widget)
     {
       gint nat_width, width, height;
 
-      gtk_size_request_get_width (GTK_SIZE_REQUEST (priv->label_widget), NULL, &nat_width);
+      gtk_widget_get_preferred_width (priv->label_widget, NULL, &nat_width);
 
-      width = widget->allocation.width;
+      width = allocation.width;
       width -= 2 * LABEL_PAD + 2 * LABEL_SIDE_PAD;
-      width -= (border_width + GTK_WIDGET (widget)->style->xthickness) * 2;
+      width -= (border_width + style->xthickness) * 2;
 
       width = MIN (width, nat_width);
 
-      gtk_size_request_get_height_for_width (GTK_SIZE_REQUEST (priv->label_widget), width,
-					     &height, NULL);
+      gtk_widget_get_preferred_height_for_width (priv->label_widget, width,
+                                                 &height, NULL);
 
-      top_margin = MAX (height, widget->style->ythickness);
+      top_margin = MAX (height, style->ythickness);
     }
   else
-    top_margin = widget->style->ythickness;
-  
-  child_allocation->x = border_width + widget->style->xthickness;
-  child_allocation->width = MAX(1, (gint)allocation->width - child_allocation->x * 2);
-  
+    top_margin = style->ythickness;
+
+  child_allocation->x = border_width + style->xthickness;
   child_allocation->y = border_width + top_margin;
-  child_allocation->height = MAX (1, ((gint)allocation->height - child_allocation->y -
-				      border_width -
-				      (gint)widget->style->ythickness));
-  
-  child_allocation->x += allocation->x;
-  child_allocation->y += allocation->y;
+  child_allocation->width = MAX (1, (gint) allocation.width - child_allocation->x * 2);
+  child_allocation->height = MAX (1, ((gint) allocation.height - child_allocation->y -
+                                      border_width - (gint) style->ythickness));
+
+  child_allocation->x += allocation.x;
+  child_allocation->y += allocation.y;
 }
 
 static void
-gtk_frame_get_size (GtkSizeRequest *request,
-		    GtkOrientation  orientation,
-		    gint           *minimum_size,
-		    gint           *natural_size)
+gtk_frame_get_preferred_size (GtkWidget      *request,
+                              GtkOrientation  orientation,
+                              gint           *minimum_size,
+                              gint           *natural_size)
 {
+  GtkFrame *frame = GTK_FRAME (request);
+  GtkFramePrivate *priv = frame->priv;
+  GtkStyle *style;
   GtkWidget *widget = GTK_WIDGET (request);
   GtkWidget *child;
-  GtkFrame *frame = GTK_FRAME (widget);
-  GtkFramePriv *priv = frame->priv;
   GtkBin *bin = GTK_BIN (widget);
   gint child_min, child_nat;
   gint minimum, natural;
   guint border_width;
 
+  style = gtk_widget_get_style (widget);
+
   if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
     {
       if (orientation == GTK_ORIENTATION_HORIZONTAL)
         {
-          gtk_size_request_get_width (GTK_SIZE_REQUEST (priv->label_widget),
-				      &child_min, &child_nat);
+          gtk_widget_get_preferred_width (priv->label_widget,
+                                          &child_min, &child_nat);
           minimum = child_min + 2 * LABEL_PAD + 2 * LABEL_SIDE_PAD;
           natural = child_nat + 2 * LABEL_PAD + 2 * LABEL_SIDE_PAD;
         }
       else
         {
-          gtk_size_request_get_height (GTK_SIZE_REQUEST (priv->label_widget),
-				       &child_min, &child_nat);
-          minimum = MAX (0, child_min - widget->style->ythickness);
-          natural = MAX (0, child_nat - widget->style->ythickness);
+          gtk_widget_get_preferred_height (priv->label_widget,
+                                           &child_min, &child_nat);
+          minimum = MAX (0, child_min - style->ythickness);
+          natural = MAX (0, child_nat - style->ythickness);
         }
     }
   else
@@ -821,15 +805,15 @@ gtk_frame_get_size (GtkSizeRequest *request,
     {
       if (orientation == GTK_ORIENTATION_HORIZONTAL)
         {
-          gtk_size_request_get_width (GTK_SIZE_REQUEST (child),
-				      &child_min, &child_nat);
+          gtk_widget_get_preferred_width (child,
+                                          &child_min, &child_nat);
           minimum = MAX (minimum, child_min);
           natural = MAX (natural, child_nat);
         }
       else
         {
-          gtk_size_request_get_height (GTK_SIZE_REQUEST (child),
-				       &child_min, &child_nat);
+          gtk_widget_get_preferred_height (child,
+                                           &child_min, &child_nat);
           minimum += child_min;
           natural += child_nat;
         }
@@ -839,13 +823,13 @@ gtk_frame_get_size (GtkSizeRequest *request,
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-      minimum += (border_width + GTK_WIDGET (widget)->style->xthickness) * 2;
-      natural += (border_width + GTK_WIDGET (widget)->style->xthickness) * 2;
+      minimum += (border_width + style->xthickness) * 2;
+      natural += (border_width + style->xthickness) * 2;
     }
   else
     {
-      minimum += (border_width + GTK_WIDGET (widget)->style->ythickness) * 2;
-      natural += (border_width + GTK_WIDGET (widget)->style->ythickness) * 2;
+      minimum += (border_width + style->ythickness) * 2;
+      natural += (border_width + style->ythickness) * 2;
     }
 
  if (minimum_size)
@@ -856,48 +840,51 @@ gtk_frame_get_size (GtkSizeRequest *request,
 }
 
 static void
-gtk_frame_get_width (GtkSizeRequest *widget,
-		     gint           *minimum_size,
-		     gint           *natural_size)
+gtk_frame_get_preferred_width (GtkWidget *widget,
+                               gint      *minimum_size,
+                               gint      *natural_size)
 {
-  gtk_frame_get_size (widget, GTK_ORIENTATION_HORIZONTAL, minimum_size, natural_size);
+  gtk_frame_get_preferred_size (widget, GTK_ORIENTATION_HORIZONTAL, minimum_size, natural_size);
 }
 
 static void
-gtk_frame_get_height (GtkSizeRequest *widget,
-		      gint           *minimum_size,
-		      gint           *natural_size)
+gtk_frame_get_preferred_height (GtkWidget *widget,
+                                gint      *minimum_size,
+                                gint      *natural_size)
 {
-  gtk_frame_get_size (widget, GTK_ORIENTATION_VERTICAL, minimum_size, natural_size);
+  gtk_frame_get_preferred_size (widget, GTK_ORIENTATION_VERTICAL, minimum_size, natural_size);
 }
 
 
 static void
-gtk_frame_get_height_for_width (GtkSizeRequest *request,
-				gint            width,
-				gint           *minimum_height,
-				gint           *natural_height)
+gtk_frame_get_preferred_height_for_width (GtkWidget *request,
+                                          gint       width,
+                                          gint      *minimum_height,
+                                          gint      *natural_height)
 {
   GtkWidget *widget = GTK_WIDGET (request);
   GtkWidget *child;
   GtkFrame *frame = GTK_FRAME (widget);
-  GtkFramePriv *priv = frame->priv;
+  GtkFramePrivate *priv = frame->priv;
   GtkBin *bin = GTK_BIN (widget);
+  GtkStyle *style;
   gint child_min, child_nat, label_width;
   gint minimum, natural;
   guint border_width;
 
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  minimum      = (border_width + GTK_WIDGET (widget)->style->ythickness) * 2;
-  natural      = (border_width + GTK_WIDGET (widget)->style->ythickness) * 2;
+  style = gtk_widget_get_style (widget);
 
-  width -= (border_width + GTK_WIDGET (widget)->style->xthickness) * 2;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  minimum      = (border_width + style->ythickness) * 2;
+  natural      = (border_width + style->ythickness) * 2;
+
+  width -= (border_width + style->xthickness) * 2;
   label_width = width - 2 * LABEL_PAD + 2 * LABEL_SIDE_PAD;
 
   if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
     {
-      gtk_size_request_get_height_for_width (GTK_SIZE_REQUEST (priv->label_widget),
-					     label_width, &child_min, &child_nat);
+      gtk_widget_get_preferred_height_for_width (priv->label_widget,
+                                                 label_width, &child_min, &child_nat);
       minimum += child_min;
       natural += child_nat;
     }
@@ -905,8 +892,8 @@ gtk_frame_get_height_for_width (GtkSizeRequest *request,
   child = gtk_bin_get_child (bin);
   if (child && gtk_widget_get_visible (child))
     {
-      gtk_size_request_get_height_for_width (GTK_SIZE_REQUEST (child),
-					     width, &child_min, &child_nat);
+      gtk_widget_get_preferred_height_for_width (child,
+                                                 width, &child_min, &child_nat);
       minimum += child_min;
       natural += child_nat;
     }
@@ -919,19 +906,11 @@ gtk_frame_get_height_for_width (GtkSizeRequest *request,
 }
 
 static void
-gtk_frame_get_width_for_height (GtkSizeRequest *widget,
-				gint       height,
-				gint      *minimum_width,
-				gint      *natural_width)
+gtk_frame_get_preferred_width_for_height (GtkWidget *widget,
+                                          gint       height,
+                                          gint      *minimum_width,
+                                          gint      *natural_width)
 {
-  gtk_size_request_get_width (widget, minimum_width, natural_width);
+  GTK_WIDGET_GET_CLASS (widget)->get_preferred_width (widget, minimum_width, natural_width);
 }
 
-static void
-gtk_frame_size_request_init (GtkSizeRequestIface *iface)
-{
-  iface->get_width            = gtk_frame_get_width;
-  iface->get_height           = gtk_frame_get_height;
-  iface->get_height_for_width = gtk_frame_get_height_for_width;
-  iface->get_width_for_height = gtk_frame_get_width_for_height;
-}
