@@ -28,14 +28,16 @@
  */
 
 #include "config.h"
-#include <string.h>
 
 #include "gtkstatusicon.h"
 
+#include <string.h>
+
 #include "gtkintl.h"
 #include "gtkiconfactory.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
+#include "gtksizerequest.h"
 #include "gtktrayicon.h"
 
 #include "gtkprivate.h"
@@ -61,8 +63,6 @@
 
 #include "gdkkeysyms.h"
 
-#include "gtkalias.h"
-
 #define BLINK_TIMEOUT 500
 
 enum
@@ -142,11 +142,6 @@ struct _GtkStatusIconPrivate
       GIcon     *gicon;
     } image_data;
 
-  GdkPixbuf    *blank_icon;
-  guint         blinking_timeout;
-
-  guint         blinking : 1;
-  guint         blink_off : 1;
   guint         visible : 1;
 };
 
@@ -170,6 +165,11 @@ static void     gtk_status_icon_screen_changed   (GtkStatusIcon  *status_icon,
 						  GdkScreen      *old_screen);
 static void     gtk_status_icon_embedded_changed (GtkStatusIcon *status_icon);
 static void     gtk_status_icon_orientation_changed (GtkStatusIcon *status_icon);
+static void     gtk_status_icon_padding_changed  (GtkStatusIcon *status_icon);
+static void     gtk_status_icon_fg_changed       (GtkStatusIcon *status_icon);
+static void     gtk_status_icon_color_changed    (GtkTrayIcon   *tray,
+                                                  GParamSpec    *pspec,
+                                                  GtkStatusIcon *status_icon);
 static gboolean gtk_status_icon_scroll           (GtkStatusIcon  *status_icon,
 						  GdkEventScroll *event);
 static gboolean gtk_status_icon_query_tooltip    (GtkStatusIcon *status_icon,
@@ -186,7 +186,6 @@ static gboolean gtk_status_icon_button_press     (GtkStatusIcon  *status_icon,
 						  GdkEventButton *event);
 static gboolean gtk_status_icon_button_release   (GtkStatusIcon  *status_icon,
 						  GdkEventButton *event);
-static void     gtk_status_icon_disable_blinking (GtkStatusIcon  *status_icon);
 static void     gtk_status_icon_reset_image_data (GtkStatusIcon  *status_icon);
 static void     gtk_status_icon_update_image    (GtkStatusIcon *status_icon);
 
@@ -282,26 +281,11 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
 							GDK_TYPE_SCREEN,
  							GTK_PARAM_READWRITE));
 
-  /**
-   * GtkStatusIcon:blinking:
-   *
-   * Whether or not the status icon is blinking.
-   *
-   * Deprecated: 2.22: This property will be removed in GTK+ 3
-   */
-  g_object_class_install_property (gobject_class,
-				   PROP_BLINKING,
-				   g_param_spec_boolean ("blinking",
-							 P_("Blinking"),
-							 P_("Whether or not the status icon is blinking"),
-							 FALSE,
-							 GTK_PARAM_READWRITE | G_PARAM_DEPRECATED));
-
   g_object_class_install_property (gobject_class,
 				   PROP_VISIBLE,
 				   g_param_spec_boolean ("visible",
 							 P_("Visible"),
-							 P_("Whether or not the status icon is visible"),
+							 P_("Whether the status icon is visible"),
 							 TRUE,
 							 GTK_PARAM_READWRITE));
 
@@ -317,7 +301,7 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
 				   PROP_EMBEDDED,
 				   g_param_spec_boolean ("embedded",
 							 P_("Embedded"),
-							 P_("Whether or not the status icon is embedded"),
+							 P_("Whether the status icon is embedded"),
 							 FALSE,
 							 GTK_PARAM_READABLE));
 
@@ -572,13 +556,15 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
    * @event: the #GdkEventScroll which triggered this signal
    *
    * The ::scroll-event signal is emitted when a button in the 4 to 7
-   * range is pressed. Wheel mice are usually configured to generate 
+   * range is pressed. Wheel mice are usually configured to generate
    * button press events for buttons 4 and 5 when the wheel is turned.
    *
    * Whether this event is emitted is platform-dependent.
    *
-   * Returns: %TRUE to stop other handlers from being invoked for the event. 
+   * Returns: %TRUE to stop other handlers from being invoked for the event.
    *   %FALSE to propagate the event further.
+   *
+   * Since: 2.16
    */
   status_icon_signals[SCROLL_EVENT_SIGNAL] =
     g_signal_new (I_("scroll_event"),
@@ -834,7 +820,7 @@ gtk_status_icon_init (GtkStatusIcon *status_icon)
   priv = G_TYPE_INSTANCE_GET_PRIVATE (status_icon, GTK_TYPE_STATUS_ICON,
 				      GtkStatusIconPrivate);
   status_icon->priv = priv;
-  
+
   priv->storage_type = GTK_IMAGE_EMPTY;
   priv->visible      = TRUE;
 
@@ -857,6 +843,16 @@ gtk_status_icon_init (GtkStatusIcon *status_icon)
 			    G_CALLBACK (gtk_status_icon_embedded_changed), status_icon);
   g_signal_connect_swapped (priv->tray_icon, "notify::orientation",
 			    G_CALLBACK (gtk_status_icon_orientation_changed), status_icon);
+  g_signal_connect_swapped (priv->tray_icon, "notify::padding",
+			    G_CALLBACK (gtk_status_icon_padding_changed), status_icon);
+  g_signal_connect_swapped (priv->tray_icon, "notify::fg-color",
+                            G_CALLBACK (gtk_status_icon_fg_changed), status_icon);
+  g_signal_connect (priv->tray_icon, "notify::error-color",
+                    G_CALLBACK (gtk_status_icon_color_changed), status_icon);
+  g_signal_connect (priv->tray_icon, "notify::warning-color",
+                    G_CALLBACK (gtk_status_icon_color_changed), status_icon);
+  g_signal_connect (priv->tray_icon, "notify::success-color",
+                    G_CALLBACK (gtk_status_icon_color_changed), status_icon);
   g_signal_connect_swapped (priv->tray_icon, "button-press-event",
 			    G_CALLBACK (gtk_status_icon_button_press), status_icon);
   g_signal_connect_swapped (priv->tray_icon, "button-release-event",
@@ -975,14 +971,8 @@ gtk_status_icon_finalize (GObject *object)
   GtkStatusIcon *status_icon = GTK_STATUS_ICON (object);
   GtkStatusIconPrivate *priv = status_icon->priv;
 
-  gtk_status_icon_disable_blinking (status_icon);
-  
   gtk_status_icon_reset_image_data (status_icon);
 
-  if (priv->blank_icon)
-    g_object_unref (priv->blank_icon);
-  priv->blank_icon = NULL;
-
 #ifdef GDK_WINDOWING_X11
   g_signal_handlers_disconnect_by_func (priv->tray_icon,
 			                gtk_status_icon_key_press, status_icon);
@@ -992,6 +982,12 @@ gtk_status_icon_finalize (GObject *object)
 			                gtk_status_icon_embedded_changed, status_icon);
   g_signal_handlers_disconnect_by_func (priv->tray_icon,
 			                gtk_status_icon_orientation_changed, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_padding_changed, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+                                        gtk_status_icon_fg_changed, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+                                        gtk_status_icon_color_changed, status_icon);
   g_signal_handlers_disconnect_by_func (priv->tray_icon,
 			                gtk_status_icon_button_press, status_icon);
   g_signal_handlers_disconnect_by_func (priv->tray_icon,
@@ -1056,9 +1052,6 @@ gtk_status_icon_set_property (GObject      *object,
     case PROP_SCREEN:
       gtk_status_icon_set_screen (status_icon, g_value_get_object (value));
       break;
-    case PROP_BLINKING:
-      gtk_status_icon_set_blinking (status_icon, g_value_get_boolean (value));
-      break;
     case PROP_VISIBLE:
       gtk_status_icon_set_visible (status_icon, g_value_get_boolean (value));
       break;
@@ -1130,9 +1123,6 @@ gtk_status_icon_get_property (GObject    *object,
     case PROP_SCREEN:
       g_value_set_object (value, gtk_status_icon_get_screen (status_icon));
       break;
-    case PROP_BLINKING:
-      g_value_set_boolean (value, gtk_status_icon_get_blinking (status_icon));
-      break;
     case PROP_VISIBLE:
       g_value_set_boolean (value, gtk_status_icon_get_visible (status_icon));
       break;
@@ -1318,37 +1308,6 @@ emit_size_changed_signal (GtkStatusIcon *status_icon,
 
 #endif
 
-static GdkPixbuf *
-gtk_status_icon_blank_icon (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-
-  if (priv->blank_icon)
-    {
-      gint width, height;
-
-      width  = gdk_pixbuf_get_width (priv->blank_icon);
-      height = gdk_pixbuf_get_height (priv->blank_icon);
-
-
-      if (width == priv->image_width && height == priv->image_height)
-	return priv->blank_icon;
-      else
-	{
-	  g_object_unref (priv->blank_icon);
-	  priv->blank_icon = NULL;
-	}
-    }
-
-  priv->blank_icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				     priv->image_width, 
-				     priv->image_height);
-  if (priv->blank_icon)
-    gdk_pixbuf_fill (priv->blank_icon, 0);
-
-  return priv->blank_icon;
-}
-
 #ifdef GDK_WINDOWING_X11
 
 static GtkIconSize
@@ -1397,30 +1356,6 @@ gtk_status_icon_update_image (GtkStatusIcon *status_icon)
   HICON prev_hicon;
 #endif
 
-  if (priv->blink_off)
-    {
-#ifdef GDK_WINDOWING_X11
-      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image),
-				 gtk_status_icon_blank_icon (status_icon));
-#endif
-#ifdef GDK_WINDOWING_WIN32
-      prev_hicon = priv->nid.hIcon;
-      priv->nid.hIcon = gdk_win32_pixbuf_to_hicon_libgtk_only (gtk_status_icon_blank_icon (status_icon));
-      priv->nid.uFlags |= NIF_ICON;
-      if (priv->nid.hWnd != NULL && priv->visible)
-	if (!Shell_NotifyIconW (NIM_MODIFY, &priv->nid))
-	  g_warning (G_STRLOC ": Shell_NotifyIcon(NIM_MODIFY) failed");
-      if (prev_hicon)
-	DestroyIcon (prev_hicon);
-#endif
-#ifdef GDK_WINDOWING_QUARTZ
-      QUARTZ_POOL_ALLOC;
-      [priv->status_item setImage:gtk_status_icon_blank_icon (status_icon)];
-      QUARTZ_POOL_RELEASE;
-#endif
-      return;
-    }
-
   switch (priv->storage_type)
     {
     case GTK_IMAGE_PIXBUF:
@@ -1499,10 +1434,9 @@ gtk_status_icon_update_image (GtkStatusIcon *status_icon)
 #ifdef GDK_WINDOWING_WIN32
 	{
 	  GdkPixbuf *pixbuf =
-	    gtk_widget_render_icon (priv->dummy_widget,
-				    priv->image_data.stock_id,
-				    GTK_ICON_SIZE_SMALL_TOOLBAR,
-				    NULL);
+	    gtk_widget_render_icon_pixbuf (priv->dummy_widget,
+                                           priv->image_data.stock_id,
+                                           GTK_ICON_SIZE_SMALL_TOOLBAR);
 
 	  prev_hicon = priv->nid.hIcon;
 	  priv->nid.hIcon = gdk_win32_pixbuf_to_hicon_libgtk_only (pixbuf);
@@ -1519,10 +1453,9 @@ gtk_status_icon_update_image (GtkStatusIcon *status_icon)
 	{
 	  GdkPixbuf *pixbuf;
 
-	  pixbuf = gtk_widget_render_icon (priv->dummy_widget,
-					   priv->image_data.stock_id,
-					   GTK_ICON_SIZE_SMALL_TOOLBAR,
-					   NULL);
+	  pixbuf = gtk_widget_render_icon_pixbuf (priv->dummy_widget,
+                                                  priv->image_data.stock_id,
+                                                  GTK_ICON_SIZE_SMALL_TOOLBAR);
 	  QUARTZ_POOL_ALLOC;
 	  [priv->status_item setImage:pixbuf];
 	  QUARTZ_POOL_RELEASE;
@@ -1658,6 +1591,7 @@ gtk_status_icon_size_allocate (GtkStatusIcon *status_icon,
   GtkStatusIconPrivate *priv = status_icon->priv;
   GtkOrientation orientation;
   gint size;
+  gint xpad, ypad;
 
   orientation = _gtk_tray_icon_get_orientation (GTK_TRAY_ICON (priv->tray_icon));
 
@@ -1666,8 +1600,10 @@ gtk_status_icon_size_allocate (GtkStatusIcon *status_icon,
   else
     size = allocation->width;
 
-  priv->image_width = allocation->width - GTK_MISC (priv->image)->xpad * 2;
-  priv->image_height = allocation->height - GTK_MISC (priv->image)->ypad * 2;
+  gtk_misc_get_padding (GTK_MISC (priv->image), &xpad, &ypad);
+
+  priv->image_width = allocation->width - xpad * 2;
+  priv->image_height = allocation->height - ypad * 2;
 
   if (priv->size != size)
     {
@@ -1696,18 +1632,86 @@ gtk_status_icon_screen_changed (GtkStatusIcon *status_icon,
 
 #ifdef GDK_WINDOWING_X11
 
+static void
+gtk_status_icon_padding_changed (GtkStatusIcon *status_icon)
+{
+  GtkStatusIconPrivate *priv = status_icon->priv;
+  GtkOrientation orientation;
+  gint padding;
+
+  orientation = _gtk_tray_icon_get_orientation (GTK_TRAY_ICON (priv->tray_icon));
+  padding = _gtk_tray_icon_get_padding (GTK_TRAY_ICON (priv->tray_icon));
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    gtk_misc_set_padding (GTK_MISC (priv->image), padding, 0);
+  else
+    gtk_misc_set_padding (GTK_MISC (priv->image), 0, padding);
+}
+
 static void
 gtk_status_icon_embedded_changed (GtkStatusIcon *status_icon)
 {
+  gtk_status_icon_padding_changed (status_icon);
   g_object_notify (G_OBJECT (status_icon), "embedded");
 }
 
 static void
 gtk_status_icon_orientation_changed (GtkStatusIcon *status_icon)
 {
+  gtk_status_icon_padding_changed (status_icon);
   g_object_notify (G_OBJECT (status_icon), "orientation");
 }
 
+static void
+gtk_status_icon_fg_changed (GtkStatusIcon *status_icon)
+{
+  GtkStatusIconPrivate *priv = status_icon->priv;
+  GdkColor color;
+
+  g_object_get (priv->tray_icon, "fg-color", &color, NULL);
+  gtk_widget_modify_fg (priv->image, GTK_STATE_NORMAL, &color);
+}
+
+static void
+gtk_status_icon_color_changed (GtkTrayIcon   *tray,
+                               GParamSpec    *pspec,
+                               GtkStatusIcon *status_icon)
+{
+  GtkStatusIconPrivate *priv = status_icon->priv;
+  const gchar *name;
+  GdkColor color;
+
+  switch (pspec->name[0])
+    {
+    case 'e':
+      name = "error";
+      break;
+    case 'w':
+      name = "warning";
+      break;
+    case 's':
+      name = "success";
+      break;
+    default:
+      name = NULL;
+      break;
+    }
+
+  if (name)
+    {
+      GdkRGBA rgba;
+
+      g_object_get (priv->tray_icon, pspec->name, &color, NULL);
+
+      rgba.red = color.red / 65535.;
+      rgba.green = color.green / 65535.;
+      rgba.blue = color.blue / 65535.;
+      rgba.alpha = 1;
+
+      gtk_widget_override_symbolic_color (priv->image, name, &rgba);
+    }
+}
+
 static gboolean
 gtk_status_icon_key_press (GtkStatusIcon  *status_icon,
 			   GdkEventKey    *event)
@@ -1717,11 +1721,11 @@ gtk_status_icon_key_press (GtkStatusIcon  *status_icon,
   state = event->state & gtk_accelerator_get_default_mod_mask ();
   keyval = event->keyval;
   if (state == 0 &&
-      (keyval == GDK_Return ||
-       keyval == GDK_KP_Enter ||
-       keyval == GDK_ISO_Enter ||
-       keyval == GDK_space ||
-       keyval == GDK_KP_Space))
+      (keyval == GDK_KEY_Return ||
+       keyval == GDK_KEY_KP_Enter ||
+       keyval == GDK_KEY_ISO_Enter ||
+       keyval == GDK_KEY_space ||
+       keyval == GDK_KEY_KP_Space))
     {
       emit_activate_signal (status_icon);
       return TRUE;
@@ -1750,16 +1754,16 @@ gtk_status_icon_button_press (GtkStatusIcon  *status_icon,
   if (handled)
     return TRUE;
 
-  if (_gtk_button_event_triggers_context_menu (event))
-    {
-      emit_popup_menu_signal (status_icon, event->button, event->time);
-      return TRUE;
-    }
-  else if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
+  if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
     {
       emit_activate_signal (status_icon);
       return TRUE;
     }
+  else if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+    {
+      emit_popup_menu_signal (status_icon, event->button, event->time);
+      return TRUE;
+    }
 
   return FALSE;
 }
@@ -2075,7 +2079,7 @@ gtk_status_icon_get_pixbuf (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-const gchar *
+G_CONST_RETURN gchar *
 gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2107,7 +2111,7 @@ gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-const gchar *
+G_CONST_RETURN gchar *
 gtk_status_icon_get_icon_name (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2228,67 +2232,6 @@ gtk_status_icon_get_screen (GtkStatusIcon *status_icon)
 #endif
 }
 
-/**
- * gtk_status_icon_set_tooltip:
- * @status_icon: a #GtkStatusIcon
- * @tooltip_text: (allow-none): the tooltip text, or %NULL
- *
- * Sets the tooltip of the status icon.
- *
- * Since: 2.10
- *
- * Deprecated: 2.16: Use gtk_status_icon_set_tooltip_text() instead.
- */
-void
-gtk_status_icon_set_tooltip (GtkStatusIcon *status_icon,
-			     const gchar   *tooltip_text)
-{
-  gtk_status_icon_set_tooltip_text (status_icon, tooltip_text);
-}
-
-static gboolean
-gtk_status_icon_blinker (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-  
-  priv->blink_off = !priv->blink_off;
-
-  gtk_status_icon_update_image (status_icon);
-
-  return TRUE;
-}
-
-static void
-gtk_status_icon_enable_blinking (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-  
-  if (!priv->blinking_timeout)
-    {
-      gtk_status_icon_blinker (status_icon);
-
-      priv->blinking_timeout =
-	gdk_threads_add_timeout (BLINK_TIMEOUT, 
-		       (GSourceFunc) gtk_status_icon_blinker, 
-		       status_icon);
-    }
-}
-
-static void
-gtk_status_icon_disable_blinking (GtkStatusIcon *status_icon)
-{
-  GtkStatusIconPrivate *priv = status_icon->priv;
-  
-  if (priv->blinking_timeout)
-    {
-      g_source_remove (priv->blinking_timeout);
-      priv->blinking_timeout = 0;
-      priv->blink_off = FALSE;
-
-      gtk_status_icon_update_image (status_icon);
-    }
-}
-
 /**
  * gtk_status_icon_set_visible:
  * @status_icon: a #GtkStatusIcon
@@ -2362,66 +2305,6 @@ gtk_status_icon_get_visible (GtkStatusIcon *status_icon)
   return status_icon->priv->visible;
 }
 
-/**
- * gtk_status_icon_set_blinking:
- * @status_icon: a #GtkStatusIcon
- * @blinking: %TRUE to turn blinking on, %FALSE to turn it off
- * 
- * Makes the status icon start or stop blinking. 
- * Note that blinking user interface elements may be problematic
- * for some users, and thus may be turned off, in which case
- * this setting has no effect.
- *
- * Since: 2.10
- *
- * Deprecated: 2.22: This function will be removed in GTK+ 3
- **/
-void
-gtk_status_icon_set_blinking (GtkStatusIcon *status_icon,
-			      gboolean       blinking)
-{
-  GtkStatusIconPrivate *priv;
-
-  g_return_if_fail (GTK_IS_STATUS_ICON (status_icon));
-
-  priv = status_icon->priv;
-
-  blinking = blinking != FALSE;
-
-  if (priv->blinking != blinking)
-    {
-      priv->blinking = blinking;
-
-      if (blinking)
-	gtk_status_icon_enable_blinking (status_icon);
-      else
-	gtk_status_icon_disable_blinking (status_icon);
-
-      g_object_notify (G_OBJECT (status_icon), "blinking");
-    }
-}
-
-/**
- * gtk_status_icon_get_blinking:
- * @status_icon: a #GtkStatusIcon
- * 
- * Returns whether the icon is blinking, see 
- * gtk_status_icon_set_blinking().
- * 
- * Return value: %TRUE if the icon is blinking
- *
- * Since: 2.10
- *
- * Deprecated: 2.22: This function will be removed in GTK+ 3
- **/
-gboolean
-gtk_status_icon_get_blinking (GtkStatusIcon *status_icon)
-{
-  g_return_val_if_fail (GTK_IS_STATUS_ICON (status_icon), FALSE);
-
-  return status_icon->priv->blinking;
-}
-
 /**
  * gtk_status_icon_is_embedded:
  * @status_icon: a #GtkStatusIcon
@@ -2446,7 +2329,7 @@ gtk_status_icon_is_embedded (GtkStatusIcon *status_icon)
 #ifdef GDK_WINDOWING_X11
   plug = GTK_PLUG (status_icon->priv->tray_icon);
 
-  if (plug->socket_window)
+  if (gtk_plug_get_embedded (plug))
     return TRUE;
   else
     return FALSE;
@@ -2481,6 +2364,7 @@ gtk_status_icon_position_menu (GtkMenu  *menu,
 			       gpointer  user_data)
 {
 #ifdef GDK_WINDOWING_X11
+  GtkAllocation allocation;
   GtkStatusIcon *status_icon;
   GtkStatusIconPrivate *priv;
   GtkTrayIcon *tray_icon;
@@ -2489,6 +2373,7 @@ gtk_status_icon_position_menu (GtkMenu  *menu,
   GtkTextDirection direction;
   GtkRequisition menu_req;
   GdkRectangle monitor;
+  GdkWindow *window;
   gint monitor_num, height, width, xoffset, yoffset;
   
   g_return_if_fail (GTK_IS_MENU (menu));
@@ -2504,30 +2389,33 @@ gtk_status_icon_position_menu (GtkMenu  *menu,
   screen = gtk_widget_get_screen (widget);
   gtk_menu_set_screen (menu, screen);
 
-  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+  window = gtk_widget_get_window (widget);
+  monitor_num = gdk_screen_get_monitor_at_window (screen, window);
   if (monitor_num < 0)
     monitor_num = 0;
   gtk_menu_set_monitor (menu, monitor_num);
 
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-  gdk_window_get_origin (widget->window, x, y);
-  
-  gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
+  gdk_window_get_origin (window, x, y);
 
+  gtk_widget_get_preferred_size (GTK_WIDGET (menu),
+                                 &menu_req, NULL);
+
+  gtk_widget_get_allocation (widget, &allocation);
   if (_gtk_tray_icon_get_orientation (tray_icon) == GTK_ORIENTATION_VERTICAL)
     {
       width = 0;
-      height = widget->allocation.height;
-      xoffset = widget->allocation.width;
+      height = allocation.height;
+      xoffset = allocation.width;
       yoffset = 0;
     }
   else
     {
-      width = widget->allocation.width;
+      width = allocation.width;
       height = 0;
       xoffset = 0;
-      yoffset = widget->allocation.height;
+      yoffset = allocation.height;
     }
 
   if (direction == GTK_TEXT_DIR_RTL)
@@ -2618,6 +2506,7 @@ gtk_status_icon_get_geometry (GtkStatusIcon    *status_icon,
 			      GtkOrientation   *orientation)
 {
 #ifdef GDK_WINDOWING_X11   
+  GtkAllocation allocation;
   GtkWidget *widget;
   GtkStatusIconPrivate *priv;
   gint x, y;
@@ -2632,11 +2521,14 @@ gtk_status_icon_get_geometry (GtkStatusIcon    *status_icon,
 
   if (area)
     {
-      gdk_window_get_origin (widget->window, &x, &y);
+      gdk_window_get_origin (gtk_widget_get_window (widget),
+                             &x, &y);
+
+      gtk_widget_get_allocation (widget, &allocation);
       area->x = x;
       area->y = y;
-      area->width = widget->allocation.width;
-      area->height = widget->allocation.height;
+      area->width = allocation.width;
+      area->height = allocation.height;
     }
 
   if (orientation)
@@ -2918,7 +2810,7 @@ gtk_status_icon_get_x11_window_id (GtkStatusIcon *status_icon)
 {
 #ifdef GDK_WINDOWING_X11
   gtk_widget_realize (GTK_WIDGET (status_icon->priv->tray_icon));
-  return GDK_WINDOW_XID (GTK_WIDGET (status_icon->priv->tray_icon)->window);
+  return GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (status_icon->priv->tray_icon)));
 #else
   return 0;
 #endif
@@ -2971,7 +2863,7 @@ gtk_status_icon_set_title (GtkStatusIcon *status_icon,
  *
  * Since: 2.18
  */
-const gchar *
+G_CONST_RETURN gchar *
 gtk_status_icon_get_title (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -3015,21 +2907,8 @@ gtk_status_icon_set_name (GtkStatusIcon *status_icon,
   priv = status_icon->priv;
 
 #ifdef GDK_WINDOWING_X11
-  if (gtk_widget_get_realized (priv->tray_icon))
-    {
-      /* gtk_window_set_wmclass() only operates on non-realized windows,
-       * so temporarily unrealize the tray here
-       */
-      gtk_widget_hide (priv->tray_icon);
-      gtk_widget_unrealize (priv->tray_icon);
-      gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
-      gtk_widget_show (priv->tray_icon);
-    }
-  else
-    gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
+  gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
 #endif
+
+  g_object_notify (G_OBJECT (status_icon), "name");
 }
-
-
-#define __GTK_STATUS_ICON_C__
-#include "gtkaliasdef.c"
