@@ -18,23 +18,23 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* need to get the prototypes of all get_type functions */
-#define GTK_ENABLE_BROKEN
-#undef GTK_DISABLE_DEPRECATED
-/* Need to get GDK_WINDOW_OBJECT */
-#undef GDK_DISABLE_DEPRECATED
 
 #include "config.h"
 
 #include <gtk/gtk.h>
-#include "gtkalias.h"
+#include "gtkspinbutton.h"
 
 #include <locale.h>
 #include <string.h>
 #include <math.h>
 
 
-/* --- test utilities --- */
+/**
+ * SECTION:gtktesting
+ * @Short_description: Utilities for testing GTK+ applications
+ * @Title: Testing
+ */
+
 /**
  * gtk_test_init:
  * @argcp: Address of the <parameter>argc</parameter> parameter of the
@@ -73,10 +73,17 @@ gtk_test_init (int    *argcp,
    * - this function could install a mock object around GtkSettings
    */
   g_setenv ("GTK_MODULES", "", TRUE);
-  g_setenv ("GTK2_RC_FILES", "/dev/null", TRUE);
   gtk_disable_setlocale();
   setlocale (LC_ALL, "C");
   g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=%s");
+
+  /* XSendEvent() doesn't work yet on XI2 events.
+   * So at the moment gdk_test_simulate_* can only
+   * send events that GTK+ understands if XI2 is
+   * disabled, bummer.
+   */
+  gdk_disable_multidevice ();
+
   gtk_init (argcp, argvp);
 }
 
@@ -84,17 +91,21 @@ static GSList*
 test_find_widget_input_windows (GtkWidget *widget,
                                 gboolean   input_only)
 {
+  GdkWindow *window;
   GList *node, *children;
   GSList *matches = NULL;
   gpointer udata;
-  gdk_window_get_user_data (widget->window, &udata);
-  if (udata == widget && (!input_only || (GDK_IS_WINDOW (widget->window) && GDK_WINDOW_OBJECT (widget->window)->input_only)))
-    matches = g_slist_prepend (matches, widget->window);
+
+  window = gtk_widget_get_window (widget);
+
+  gdk_window_get_user_data (window, &udata);
+  if (udata == widget && (!input_only || (GDK_IS_WINDOW (window) && gdk_window_is_input_only (GDK_WINDOW (window)))))
+    matches = g_slist_prepend (matches, window);
   children = gdk_window_get_children (gtk_widget_get_parent_window (widget));
   for (node = children; node; node = node->next)
     {
       gdk_window_get_user_data (node->data, &udata);
-      if (udata == widget && (!input_only || (GDK_IS_WINDOW (node->data) && GDK_WINDOW_OBJECT (node->data)->input_only)))
+      if (udata == widget && (!input_only || (GDK_IS_WINDOW (node->data) && gdk_window_is_input_only (GDK_WINDOW (node->data)))))
         matches = g_slist_prepend (matches, node->data);
     }
   return g_slist_reverse (matches);
@@ -190,14 +201,18 @@ gtk_test_spin_button_click (GtkSpinButton  *spinner,
                             guint           button,
                             gboolean        upwards)
 {
+  GdkWindow *panel;
   gboolean b1res = FALSE, b2res = FALSE;
-  if (spinner->panel)
+
+  panel = _gtk_spin_button_get_panel (spinner);
+
+  if (panel)
     {
-      gint width, height, pos;
-      gdk_drawable_get_size (spinner->panel, &width, &height);
-      pos = upwards ? 0 : height - 1;
-      b1res = gdk_test_simulate_button (spinner->panel, width - 1, pos, button, 0, GDK_BUTTON_PRESS);
-      b2res = gdk_test_simulate_button (spinner->panel, width - 1, pos, button, 0, GDK_BUTTON_RELEASE);
+      gint width, pos;
+      width = gdk_window_get_width (panel);
+      pos = upwards ? 0 : gdk_window_get_height (panel) - 1;
+      b1res = gdk_test_simulate_button (panel, width - 1, pos, button, 0, GDK_BUTTON_PRESS);
+      b2res = gdk_test_simulate_button (panel, width - 1, pos, button, 0, GDK_BUTTON_RELEASE);
     }
   return b1res && b2res;
 }
@@ -268,13 +283,19 @@ widget_geo_dist (GtkWidget *a,
                  GtkWidget *b,
                  GtkWidget *base)
 {
+  GtkAllocation allocation;
   int ax0, ay0, ax1, ay1, bx0, by0, bx1, by1, xdist = 0, ydist = 0;
+
+  gtk_widget_get_allocation (a, &allocation);
   if (!gtk_widget_translate_coordinates (a, base, 0, 0, &ax0, &ay0) ||
-      !gtk_widget_translate_coordinates (a, base, a->allocation.width, a->allocation.height, &ax1, &ay1))
+      !gtk_widget_translate_coordinates (a, base, allocation.width, allocation.height, &ax1, &ay1))
     return -G_MAXINT;
+
+  gtk_widget_get_allocation (b, &allocation);
   if (!gtk_widget_translate_coordinates (b, base, 0, 0, &bx0, &by0) ||
-      !gtk_widget_translate_coordinates (b, base, b->allocation.width, b->allocation.height, &bx1, &by1))
+      !gtk_widget_translate_coordinates (b, base, allocation.width, allocation.height, &bx1, &by1))
     return +G_MAXINT;
+
   if (bx0 >= ax1)
     xdist = bx0 - ax1;
   else if (ax0 >= bx1)
@@ -283,6 +304,7 @@ widget_geo_dist (GtkWidget *a,
     ydist = by0 - ay1;
   else if (ay0 >= by1)
     ydist = ay0 - by1;
+
   return xdist + ydist;
 }
 
@@ -325,7 +347,7 @@ gtk_test_find_sibling (GtkWidget *base_widget,
   /* find all sibling candidates */
   while (tmpwidget)
     {
-      tmpwidget = tmpwidget->parent;
+      tmpwidget = gtk_widget_get_parent (tmpwidget);
       siblings = g_list_concat (siblings, test_list_descendants (tmpwidget, widget_type));
     }
   /* sort them by distance to base_widget */
@@ -484,7 +506,7 @@ gtk_test_text_get (GtkWidget *widget)
 /**
  * gtk_test_create_widget
  * @widget_type: a valid widget type.
- * @first_property_name: Name of first property to set or %NULL
+ * @first_property_name: (allow-none): Name of first property to set or %NULL
  * @Varargs: value to set the first property to, followed by more
  *    name-value pairs, terminated by %NULL
  *
@@ -529,7 +551,8 @@ try_main_quit (void)
 static int
 test_increment_intp (int *intp)
 {
-  *intp += 1;
+  if (intp != NULL)
+    *intp += 1;
   return 1; /* TRUE in case we're connected to event signals */
 }
 
@@ -653,6 +676,3 @@ gtk_test_register_all_types (void)
       *tp = 0;
     }
 }
-
-#define __GTK_TEST_UTILS_C__
-#include "gtkaliasdef.c"
