@@ -42,6 +42,10 @@ GType gtk_cell_renderer_mode_get_type (void) G_GNUC_CONST;
 GType gtk_cell_renderer_accel_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_CELL_RENDERER_ACCEL_MODE (gtk_cell_renderer_accel_mode_get_type ())
 
+/* enumerations from "gtkcssprovider.h" */
+GType gtk_css_provider_error_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_CSS_PROVIDER_ERROR (gtk_css_provider_error_get_type ())
+
 /* enumerations from "gtkdebug.h" */
 GType gtk_debug_flag_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DEBUG_FLAG (gtk_debug_flag_get_type ())
@@ -91,8 +95,6 @@ GType gtk_menu_direction_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MENU_DIRECTION_TYPE (gtk_menu_direction_type_get_type ())
 GType gtk_message_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MESSAGE_TYPE (gtk_message_type_get_type ())
-GType gtk_metric_type_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_METRIC_TYPE (gtk_metric_type_get_type ())
 GType gtk_movement_step_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_MOVEMENT_STEP (gtk_movement_step_get_type ())
 GType gtk_scroll_step_get_type (void) G_GNUC_CONST;
@@ -161,6 +163,16 @@ GType gtk_drag_result_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_DRAG_RESULT (gtk_drag_result_get_type ())
 GType gtk_size_request_mode_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SIZE_REQUEST_MODE (gtk_size_request_mode_get_type ())
+GType gtk_scrollable_policy_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_SCROLLABLE_POLICY (gtk_scrollable_policy_get_type ())
+GType gtk_state_flags_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_STATE_FLAGS (gtk_state_flags_get_type ())
+GType gtk_region_flags_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_REGION_FLAGS (gtk_region_flags_get_type ())
+GType gtk_junction_sides_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_JUNCTION_SIDES (gtk_junction_sides_get_type ())
+GType gtk_border_style_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_BORDER_STYLE (gtk_border_style_get_type ())
 
 /* enumerations from "gtkfilechooser.h" */
 GType gtk_file_chooser_action_get_type (void) G_GNUC_CONST;
