@@ -32,6 +32,7 @@
 #define __GTK_WINDOW_H__
 
 
+#include <gtk/gtkapplication.h>
 #include <gtk/gtkaccelgroup.h>
 #include <gtk/gtkbin.h>
 
@@ -47,73 +48,18 @@ G_BEGIN_DECLS
 #define GTK_IS_WINDOW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_WINDOW))
 #define GTK_WINDOW_GET_CLASS(obj)       (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_WINDOW, GtkWindowClass))
 
-
+typedef struct _GtkWindowPrivate      GtkWindowPrivate;
 typedef struct _GtkWindowClass        GtkWindowClass;
 typedef struct _GtkWindowGeometryInfo GtkWindowGeometryInfo;
 typedef struct _GtkWindowGroup        GtkWindowGroup;
 typedef struct _GtkWindowGroupClass   GtkWindowGroupClass;
+typedef struct _GtkWindowGroupPrivate GtkWindowGroupPrivate;
 
 struct _GtkWindow
 {
   GtkBin bin;
 
-  gchar *GSEAL (title);
-  gchar *GSEAL (wmclass_name);
-  gchar *GSEAL (wmclass_class);
-  gchar *GSEAL (wm_role);
-
-  GtkWidget *GSEAL (focus_widget);
-  GtkWidget *GSEAL (default_widget);
-  GtkWindow *GSEAL (transient_parent);
-  GtkWindowGeometryInfo *GSEAL (geometry_info);
-  GdkWindow *GSEAL (frame);
-  GtkWindowGroup *GSEAL (group);
-
-  guint16 GSEAL (configure_request_count);
-  guint GSEAL (allow_shrink) : 1;
-  guint GSEAL (resizable) : 1;
-  guint GSEAL (configure_notify_received) : 1;
-  /* The following flags are initially TRUE (before a window is mapped).
-   * They cause us to compute a configure request that involves
-   * default-only parameters. Once mapped, we set them to FALSE.
-   * Then we set them to TRUE again on unmap (for position)
-   * and on unrealize (for size).
-   */
-  guint GSEAL (need_default_position) : 1;
-  guint GSEAL (need_default_size) : 1;
-  guint GSEAL (position) : 3;
-  guint GSEAL (type) : 4; /* GtkWindowType */ 
-  guint GSEAL (has_user_ref_count) : 1;
-  guint GSEAL (has_focus) : 1;
-
-  guint GSEAL (modal) : 1;
-  guint GSEAL (destroy_with_parent) : 1;
-  
-  guint GSEAL (has_frame) : 1;
-
-  /* gtk_window_iconify() called before realization */
-  guint GSEAL (iconify_initially) : 1;
-  guint GSEAL (stick_initially) : 1;
-  guint GSEAL (maximize_initially) : 1;
-  guint GSEAL (decorated) : 1;
-  
-  guint GSEAL (type_hint) : 3; /* GdkWindowTypeHint if the hint is one of the original eight. If not, then
-				* it contains GDK_WINDOW_TYPE_HINT_NORMAL
-				*/
-  guint GSEAL (gravity) : 5; /* GdkGravity */
-
-  guint GSEAL (is_active) : 1;
-  guint GSEAL (has_toplevel_focus) : 1;
-  
-  guint GSEAL (frame_left);
-  guint GSEAL (frame_top);
-  guint GSEAL (frame_right);
-  guint GSEAL (frame_bottom);
-
-  guint GSEAL (keys_changed_handler);
-  
-  GdkModifierType GSEAL (mnemonic_modifier);
-  GdkScreen      *GSEAL (screen);
+  GtkWindowPrivate *priv;
 };
 
 struct _GtkWindowClass
@@ -130,15 +76,8 @@ struct _GtkWindowClass
   void     (* activate_focus)          (GtkWindow       *window);
   void     (* activate_default)        (GtkWindow       *window);
 
-  /* as of GTK+ 2.12 the "move-focus" signal has been moved to GtkWidget,
-   * so this is merley a virtual function now. Overriding it in subclasses
-   * continues to work though.
-   */
-  void     (* move_focus)              (GtkWindow       *window,
-                                        GtkDirectionType direction);
-  
-  void	   (*keys_changed)	       (GtkWindow	*window);
-  
+  void	   (* keys_changed)	       (GtkWindow	*window);
+
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
@@ -157,7 +96,7 @@ struct _GtkWindowGroup
 {
   GObject parent_instance;
 
-  GSList *GSEAL (grabs);
+  GtkWindowGroupPrivate *priv;
 };
 
 struct _GtkWindowGroupClass
@@ -171,12 +110,6 @@ struct _GtkWindowGroupClass
   void (*_gtk_reserved4) (void);
 };
 
-#ifdef G_OS_WIN32
-/* Reserve old names for DLL ABI backward compatibility */
-#define gtk_window_set_icon_from_file gtk_window_set_icon_from_file_utf8
-#define gtk_window_set_default_icon_from_file gtk_window_set_default_icon_from_file_utf8
-#endif
-
 GType      gtk_window_get_type                 (void) G_GNUC_CONST;
 GtkWidget* gtk_window_new                      (GtkWindowType        type);
 void       gtk_window_set_title                (GtkWindow           *window,
@@ -377,6 +310,14 @@ void     gtk_window_get_position     (GtkWindow   *window,
                                       gint        *root_y);
 gboolean gtk_window_parse_geometry   (GtkWindow   *window,
                                       const gchar *geometry);
+
+void gtk_window_set_default_geometry (GtkWindow *window,
+                                      gint       width,
+                                      gint       height);
+void gtk_window_resize_to_geometry   (GtkWindow *window,
+                                      gint       width,
+                                      gint       height);
+
 GtkWindowGroup *gtk_window_get_group (GtkWindow   *window);
 gboolean gtk_window_has_group        (GtkWindow   *window);
 
@@ -396,63 +337,23 @@ void             gtk_window_group_remove_window (GtkWindowGroup     *window_grou
 					         GtkWindow          *window);
 GList *          gtk_window_group_list_windows  (GtkWindowGroup     *window_group);
 
+GtkWidget *      gtk_window_group_get_current_grab (GtkWindowGroup *window_group);
 GtkWidget *      gtk_window_group_get_current_device_grab (GtkWindowGroup *window_group,
                                                            GdkDevice      *device);
 
+GtkApplication *gtk_window_get_application      (GtkWindow          *window);
+void            gtk_window_set_application      (GtkWindow          *window,
+                                                 GtkApplication     *application);
 
-/* --- internal functions --- */
-void            _gtk_window_internal_set_focus (GtkWindow *window,
-						GtkWidget *focus);
-void            gtk_window_remove_embedded_xid (GtkWindow       *window,
-						GdkNativeWindow  xid);
-void            gtk_window_add_embedded_xid    (GtkWindow       *window,
-						GdkNativeWindow  xid);
-void            _gtk_window_reposition         (GtkWindow *window,
-						gint       x,
-						gint       y);
-void            _gtk_window_constrain_size     (GtkWindow *window,
-						gint       width,
-						gint       height,
-						gint      *new_width,
-						gint      *new_height);
-GtkWidget      *gtk_window_group_get_current_grab (GtkWindowGroup *window_group);
-void            _gtk_window_group_add_device_grab    (GtkWindowGroup   *window_group,
-                                                      GtkWidget        *widget,
-                                                      GdkDevice        *device,
-                                                      gboolean          block_others);
-void            _gtk_window_group_remove_device_grab (GtkWindowGroup   *window_group,
-                                                      GtkWidget        *widget,
-                                                      GdkDevice        *device);
 
-gboolean        _gtk_window_group_widget_is_blocked_for_device (GtkWindowGroup *window_group,
-                                                                GtkWidget      *widget,
-                                                                GdkDevice      *device);
-
-void            _gtk_window_set_has_toplevel_focus (GtkWindow *window,
-						    gboolean   has_toplevel_focus);
-void            _gtk_window_unset_focus_and_default (GtkWindow *window,
-						     GtkWidget *widget);
-
-void            _gtk_window_set_is_active          (GtkWindow *window,
-						    gboolean   is_active);
-
-void            _gtk_window_set_is_toplevel        (GtkWindow *window,
-						    gboolean   is_toplevel);
-
-typedef void (*GtkWindowKeysForeachFunc) (GtkWindow      *window,
-					  guint           keyval,
-					  GdkModifierType modifiers,
-					  gboolean        is_mnemonic,
-					  gpointer        data);
-
-void _gtk_window_keys_foreach (GtkWindow               *window,
-			       GtkWindowKeysForeachFunc func,
-			       gpointer                 func_data);
-
-/* --- internal (GtkAcceleratable) --- */
-gboolean	_gtk_window_query_nonaccels	(GtkWindow	*window,
-						 guint		 accel_key,
-						 GdkModifierType accel_mods);
+/* Window grips
+ */
+void     gtk_window_set_has_resize_grip    (GtkWindow    *window,
+                                            gboolean      value);
+gboolean gtk_window_get_has_resize_grip    (GtkWindow    *window);
+gboolean gtk_window_resize_grip_is_visible (GtkWindow    *window);
+gboolean gtk_window_get_resize_grip_area   (GtkWindow    *window,
+                                            GdkRectangle *rect);
 
 G_END_DECLS
 
